#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .devdoo_services import DevdooServices
from .devdoo_database import DevdooDatabase
from console import Console
from .devdoo_brokers import DevdooBrokers
from .database import DataBase
from .datafilter import DataFilter
from .datarest import DataRest
from .datarest_db import DataRestDB
from .datatypes import DataTypes
from .service import Service
from .service_db import ServiceDB
from .status import Status
from .validate import Validate



#docker exec -it 4551555545454 python ./pastas/add_services.py


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
from devdoo.datatypes import DataTypes


class DecodeTypes:
    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, status):
        self.status = status

    # --------------------------------
    # action
    # --------------------------------
    # Verifica se a action é do tipo string
    # caso contrário registra um erro e retorna None
    def action(self, value):
        # Verifica se o action é do tipo string
        if not DataTypes.is_string(value):
            self.status.error("SERVICE_INVALID", None, ["action", DataTypes.to_str(value)])
            return None
        return value

    # --------------------------------
    # connection_id
    # --------------------------------
    # Verifica se connection_id é um identificador válido
    # caso contrário registra um erro e retorna None
    def connection_id(self, value):
        # Verifica se o action é do tipo string
        if not re.match("^([0-9a-f]{24})$", value, re.IGNORECASE):
            self.status.error("SERVICE_INVALID", None, ["connection_id", DataTypes.to_str(value)])
        return value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import zmq
from datatypes import DataTypes
from console import Console
from status import Status
from zmq.sugar.socket import Socket

import yaml
import os
import sys


class DevdooBrokers(object):
    # --------------------------------
    # Error
    # --------------------------------
    # TODO:: Remover classe de error
    class Error(Exception):
        pass

    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self):

        # Inicia os tipos de dados
        self.id = None
        self.ip = None
        self.type = None
        self.service_id = None

        self.data_types = DataTypes()
        self.console = Console()
        self.status = Status()
        self.config = self.__config()

        # Inicializa variável de conexão
        self.socket_backend = None

        if self.config is not None:
            self.id = self.config["id"]
            self.ip = self.config["ip"]
            self.type = self.config["type"]
            self.service_id = self.config["service_id"]

            # Cria contexto ZMQ
            self.context = zmq.Context()

            # Inicializa o poll set
            self.poller = zmq.Poller()

            self.__start()

        else:
            # TODO:: Enviar para o servidor console a informação de falha de configuração
            print DataTypes.to_str( "AAA - FALHA NA INICIALIZACAO DO DEVDOO-BROKER-SERVICE")
            print "ERROR", self.status.to_list()

    # --------------------------------
    # __start
    # --------------------------------
    def __start(self):
        broker_endpoint = "tcp://*:" + self.data_types.to_str(self.service_id)

        # Abre porta para o Restify (frontend)
        self.socket_frontend = self.context.socket(zmq.ROUTER)
        self.socket_frontend.bind(broker_endpoint)
        self.poller.register(self.socket_frontend, zmq.POLLIN)

        if self.__is_start_services() is True:
            self.run()
        else:
            print "BBB - FALHA NA INICIALIZAÇÃO DO DEVDOO-BROKER-SERVICE"


    # --------------------------------
    # __is_start_services
    # --------------------------------
    def __is_start_services(self):
        status = False
        list_services = self.console.services_config(self.id)

        if len(list_services) > 0:
            # Abre portas para os Serviços
            self.available_services = {}
            for item in list_services:
                # Define o endpoint do serviço
                service_id = self.data_types.to_str(item["service_id"])
                endpoint = "tcp://*:" + service_id

                print "endpoint", endpoint

                # Abre portas para os Serviços no backend
                socket_backend = self.context.socket(zmq.DEALER)
                socket_backend.bind(endpoint)

                # TODO:: Implementar Lista de serviços do mesmo tipo
                # TODO:: Aprimorar registro do serviço no stack
                # Armazena a conexão do backend no stack
                self.available_services[service_id] = socket_backend

            if len(self.available_services.keys()) > 0:
                status = True

        return status

    # --------------------------------
    # operation
    # --------------------------------
    # Permuta pacotes entre o frontend e o backend
    #
    # TODO:: Verificar se existe um serviço para o service_id solicitado antes de tentar pegar na lista
    # TODO:: Verificar se o serviço está disponível antes de tentar fazer conexão
    # TODO:: Verificar se a mensagem foi corretamente convertida de string para dic antes de tentar pegar um elemento
    # TODO:: Procurar formas de desregistrar sockets que não obtiveram respostas dentro do tempo limite
    def operation(self):
        # Poller de sockets
        socks = dict(self.poller.poll())

        # Verifica se o socket chegou do REST ou Serviço
        if socks.get(self.socket_frontend) == zmq.POLLIN:
            # Recebe mensagem do Restify
            message = self.socket_frontend.recv_multipart()

            # Recupera o socket de conexao do serviço
            self.socket_backend, service_id = self.__find_socket(message)

            # Verifica se o socket é valido
            if type(self.socket_backend) == Socket:
                print "SOCKT VALIDO", service_id
                # Registra o socket no poller ZMQ para dar acesso aos workers
                self.poller.register(self.socket_backend, zmq.POLLIN)

                # Envia ao Service Work a mensagem com pacote de dados
                self.socket_backend.send_multipart(message)
            else:
                print "FALHOU, VOLTOU PARA REST"
                # Caso contrario retorna para o cliente sem realizar nenhuma alteração
                self.socket_frontend.send_multipart(message)

        # Verifica se o socket retornou do Service Worker
        if socks.get(self.socket_backend) == zmq.POLLIN:
            # Recebe do Service Worker
            message = self.socket_backend.recv_multipart()

            # Retorna ao Restify
            self.socket_frontend.send_multipart(message)
            self.poller.unregister(self.socket_backend)

    # --------------------------------
    # process
    # --------------------------------
    # Assinatura do método process para sobrescrita
    #
    def process(self, data):
        raise DevdooBrokerService.Error("`process` should be overriden")

    # --------------------------------
    # run
    # --------------------------------
    # Executa a operação em um loop infinito
    #
    def run(self):
        while True:
            try:
                self.operation()
            except Exception as inst:
                print "BROKER::RUN::ERROR", inst.args[0]

    # --------------------------------
    # __find_socket
    # --------------------------------
    def __find_socket(self, message):
        # Verifica se mensagem é string
        if (type(message) == list) and len(message) >= 3:
            # Converte mensagem para json

            # Converte a mensagem recebida do servidor rest para objeto
            package = DataTypes.to_object(message[2])

            # Verifica se o pacote de mensagem é do tipo objeto válido
            if DataTypes.is_object(package):
                # Verifica se no pacote existe o identificador do serviço
                if "service_id" in package.keys():
                    service_id = self.data_types.to_str(package["service_id"])
                    if service_id in self.available_services.keys():
                        return self.available_services[self.data_types.to_str(package["service_id"])], self.data_types.to_str(package["service_id"])

            # TODO:: Registra erro de serviço não encontrado
            return None, None

    # --------------------------------
    # __config
    # --------------------------------
    # Recupera informações de configurações do broker
    def __config(self):
        # Tenta pegar configurações docker
        devdoo_config = os.environ.get('devdoo')

        # Caso não tenha sucesso
        if not devdoo_config:
            # Tenta pegar configurações de arquivo local
            devdoo_config = self.__config_local()

        if DataTypes.is_service_config(devdoo_config):
            return devdoo_config
        else:
            self.status.error("CONFIGURATION_SERVICE", None, ["Arquivo de configuracao invalido"])
            return None

    # --------------------------------
    # __config_local
    # --------------------------------
    # Recupera informações de configuração em arquivo local
    def __config_local(self):
        config_local = None
        file_open = None

        if len(sys.argv) > 1:
            # Recupera o endereço do arquivo de configuração
            path_file = str(sys.argv[1])
            # Verifica se o endereço do arquivo é válido
            if (type(path_file) == str) and (".yml" in path_file):
                try:
                    # Abre o arquivo
                    file_yml = open(path_file, "r")
                    # Converte o arquivo em objeto
                    file_open = yaml.load(file_yml)
                except Exception as inst:
                    self.status.error("CONFIGURATION_SERVICE", None,
                                      ["Arquivo de configuracao nao encontrado"])
                    config_local = None

            # Verifica se o arquivo é válido e se existe configurações de serviços nele
            if (file_open is not None) and ("services" in file_open.keys()):
                # Processa o arquivo
                for item in file_open["services"]:
                    services = file_open["services"][item]
                    if "environment" in services:
                        environment = services["environment"]
                        if "devdoo" in environment:
                            devdoo = environment["devdoo"]
                            devdoo = DataTypes.to_object(devdoo)
                            if DataTypes.is_object(devdoo):
                                # Recupera as configurações devdoo necessárias
                                config_local = devdoo
                            else:
                                self.status.error("CONFIGURATION_SERVICE", None,
                                                  ["Arquivo de configuracao nao e um 'object' valido"])
                        self.status.error("CONFIGURATION_SERVICE", None,
                                          ["Arquivo de configuracao encontrou falhas no grupo 'devdoo'"])
                    else:
                        self.status.error("CONFIGURATION_SERVICE", None,
                                          ["Arquivo de configuracao encontrou falhas no grupo 'environment'"])
            else:
                self.status.error("CONFIGURATION_SERVICE", None,
                                  ["Arquivo de configuracao não possui o grupo 'services'"])
        else:
            self.status.error("CONFIGURATION_SERVICE", None,
                              ["Nao foi indicado nenhum endereco de arquivo de configuracao"])
        return config_local



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datatypes import DataTypes


# TODO:: Refatorar completamente a classe DataBase
class DataBase:
    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self):
        self.__data_types = DataTypes()

    # --------------------------------
    # fields_find
    # --------------------------------
    # Prepara a lista de campos que pode ser incluída na base de dados
    #
    def fields_find(self, list_scheme):
        return self.__fields_default(list_scheme)

    # --------------------------------
    # fields_insert
    # --------------------------------
    # Prepara a lista de campos que pode ser incluída na base de dados
    #
    def fields_insert(self, body, list_scheme):
        list_scheme_clean = {}
        body_field_errors = []

        # Recupera a lista de campos que pode ser adicionada na base de dados
        # Recupera a lista de campos que foram encontrados conflitos em modo desenvolvimento de microserviços
        fields_default, dev_fields_errors = self.__fields_default(list_scheme)

        # Verifica se o campo tem valor default
        for item_name in list_scheme:
            # Pega um esquema da lista
            item_scheme = list_scheme[item_name]
            # Verifica se o campo tem valor default ou é requerido
            if self.__has_default(item_scheme) or self.__required(item_scheme):
                # Registra o dot field name do campo
                item_scheme["field"] = item_name
                # Verifica se o valor do campo não é vazio
                # Registra o valor do campo
                item_scheme["value"] = self.__check_empty(item_scheme, self.__default(item_scheme))
                # Registra o schema válido na lista de esquema limpa
                list_scheme_clean[item_name] = item_scheme

        # Processa todos os campos de body para colocar em letra minuscula e criar lista de campos validos
        for item_body_field in body:
            # Processa o nome do campo recebido no body
            body_field = item_body_field.replace(' ', '_').lower().strip()

            # Verifica se o campo recebido via body esta configurado no esquema
            item_name, item_scheme = self.__check_body_item_scheme(body_field, list_scheme)

            # Caso o campo recebido via body está registrado no esquema então adiciona na lista de esquema limpa
            if item_scheme is not None:
                # Registra o dot field name do campo
                item_scheme["field"] = item_name
                # Verifica se o valor do campo não é vazio
                # Registra o valor do campo
                item_scheme["value"] = self.__check_empty(item_scheme, body[item_body_field])
                # Registra o schema válido na lista de esquema limpa
                list_scheme_clean[item_name] = item_scheme

            # Caso o campo nao esteja registrado no scheme, registra na lista de campos com erro
            if item_body_field not in fields_default:
                body_field_errors.append(item_body_field)

        return list_scheme_clean, body_field_errors, dev_fields_errors

    # --------------------------------
    # fields_update
    # --------------------------------
    def fields_update(self, body, list_scheme):
        fields_default, dev_fields_errors = self.__fields_default(list_scheme)

        list_scheme_clean = {}
        body_field_errors = []

        # Processa todos os campos de body para colocar em letra minuscula e criar lista de campos validos
        for item_body_field in body:
            body_field = item_body_field.replace(' ', '_').lower().strip()
            item_name, item_scheme = self.__check_body_item_scheme(body_field, list_scheme)

            if item_scheme is not None:
                item_scheme["field"] = item_name
                item_scheme["value"] = self.__check_empty(item_scheme, body[item_body_field])
                list_scheme_clean[item_name] = item_scheme

            # Caso o campo nao esteja registrado no scheme, registra mensagem de error
            if item_body_field not in fields_default:
                body_field_errors.append(item_body_field)

        return list_scheme_clean, body_field_errors, dev_fields_errors

    # --------------------------------
    # __check_body_item_scheme
    # --------------------------------
    # Verifica se o campo recebido via body esta configurado no esquema
    @staticmethod
    def __check_body_item_scheme(body_field, list_scheme):
        for item_scheme in list_scheme:
            if item_scheme == body_field:
                return item_scheme, list_scheme[item_scheme]
        return None, None

    # --------------------------------
    # __check_dev_fields_default
    # --------------------------------
    # Verifica se existe conflito de nomes de campos em modo desenvolvimento de microserviços
    # Não permite incluir na lista campos pai que possuem campos filhos que podem sobrepor informações
    @staticmethod
    def __check_dev_fields_default(fields_default):
        fields_clean = []
        fields_errors = {}

        # Processa a lista de campos default
        for item_check in fields_default:
            # Retira um item da lista para comparar com outros itens da mesma lista
            for item_field in fields_default:
                # Caso um campo esteja em conflito com outros campos é adicionado na lista de erros
                if item_check + '.' in item_field:
                    fields_errors[item_check] = True

        # Gera a lista de campos default sem os campos que foram encontrados conflitos
        for item_field in fields_default:
            if item_field not in fields_errors.keys():
                fields_clean.append(item_field)

        return fields_clean, fields_errors.keys()

    # --------------------------------
    # __check_empty
    # --------------------------------
    # Verifica se o valor do campo não está vazio ou é do tipo js null
    def __check_empty(self, item_scheme, value):
        # Verifica se o campo é diferente de string e se não está vazio
        if item_scheme["type"] != 'string' and (len(self.__data_types.to_str(value)) == 0 or value == ''):
            value = '__empty__'

        # Verifica se o valor do campo e do tipo null
        elif value == '__none__' or value == '__NONE__':
            value = None

        return value

    # --------------------------------
    # __default
    # --------------------------------
    @staticmethod
    def __default(item_scheme):
        value = ""
        if 'default' in item_scheme.keys():
            value = item_scheme['default']
        return value

    # --------------------------------
    # fields_default
    # --------------------------------
    # Cria lista de campos default a partir da lista de schemas
    def __fields_default(self, list_scheme):
        # Adiciona o campo id como campo default para todas asinterações com a base de dados
        fields_default = ["_id", "id"]

        # Cria lista de campos default da api
        for item_scheme in list_scheme:
            fields_default.append(item_scheme)

        return self.__check_dev_fields_default(fields_default)

    # --------------------------------
    # __has_default
    # --------------------------------
    @staticmethod
    def __has_default(item_scheme):
        has_default = False
        if 'default' in item_scheme.keys():
            has_default = True
        return has_default

    # --------------------------------
    # __required
    # --------------------------------
    @staticmethod
    def __required(item_scheme):
        value = False
        if 'required' in item_scheme.keys():
            value = item_scheme['required']
        return value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import zmq
from service import Service


# TODO:: Implementar autoconfiguração do serviço, comunicação com console admin
class DevdooServices(object):
    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self):
        # Define parâmetros globais
        self.actions = service_config["actions"]
        self.id = service_config["id"]
        self.name = service_config["name"]
        self.schemes = service_config["schemes"]
        self.custom_service = None

        # Define endpoins de conexão
        self.broker_rest_endpoint = service_config["broker_rest_endpoint"]
        self.broker_db_endpoint = service_config["broker_database_endpoint"]
        context = zmq.Context()

        # Adaptador de conexão broker
        self.socket_broker_rest = context.socket(zmq.REP)
        self.socket_broker_rest.connect(self.broker_rest_endpoint)

        # Adaptador de conexão worker
        self.socket_broker_database = context.socket(zmq.REQ)
        self.socket_broker_database.connect(self.broker_db_endpoint)

        # Serviço a ser executado
        self.service = None

    # --------------------------------
    # console_service_list
    # --------------------------------
    # Obtém da console a lista de configuração
    #
    # TODO:: Remover console_service_list, implementar sistema de autoconfiguração
    def console_service_list(self, server_console, service_id):
        return server_console[service_id]

    # --------------------------------
    # operation
    # --------------------------------
    # Executa a ação do Serviço
    #
    # TODO:: Implementar melhorias na verificação de disponibilidade do serviço para serguir adiante, chegando todos os tipos que serão enviados
    def operation(self):
        # Recebe do mensagem do servidor cliente
        message = self.socket_broker_rest.recv()

        # Inicia o Serviço
        self.process(message)

        # Executa a ação do Serviço
        self.service.action()

        # Verifica se o servico está pronto
        if self.service.ready():
            # Envia mensagem para o servidor de banco de dados
            self.socket_broker_database.send(self.service.send_database())

            # Recebe mensagem do servidor de banco de dados
            msg = self.socket_broker_database.recv()

            # Envia mensagem de retorno para o cliente
            self.socket_broker_rest.send(msg)

        # Em caso de falha
        else:
            # Nao processa a operação e envia mensagem de retorno para o cliente
            self.socket_broker_rest.send(self.service.send_error())

    # --------------------------------
    # process
    # --------------------------------
    # Inicia o Serviço
    #
    # TODO:: Mudar a forma como deverá ser configurado o serviço
    def process(self, message):
        self.service = Service(message, self.id, self.name, self.schemes, self.actions, self.custom_service)

    # --------------------------------
    # run
    # --------------------------------
    # Executa a operação em um loop infinito
    #
    # TODO:: Implementar ping de verificação de disponibilidade do serviço
    def run(self):
        while True:
            try:
                self.operation()
            except Exception as inst:
                print "API::RUN::ERROR", inst.args[0]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import time
import datetime
import unicodedata

from bson import Int64
from bson.json_util import loads
from bson.timestamp import Timestamp
from bson.objectid import ObjectId

# TODO:: Testar possivel falha em str(inst.args)
class DataTypes:
    def __init__(self):
        pass

    # --------------------------------
    # is_array
    # --------------------------------
    # Verifica se é um array
    # Retorna boolean
    #
    @staticmethod
    def is_array(value):
        return type(value) == list

    # --------------------------------
    # is_boolean
    # --------------------------------
    # Verifica se é boleano
    # Retorna boolean
    #
    @staticmethod
    def is_boolean(value):
        return type(value) == bool

    # --------------------------------
    # is_service_config
    # --------------------------------
    # Verifica se o objeto de configuração de serviços é válido
    @staticmethod
    def is_service_config(configs):
        keys = ["id", "type", "service_id", "ip"]
        status = False

        if type(configs) == dict:
            status = True
            for item in keys:
                if item not in configs.keys():
                    status = False
        return status


    # --------------------------------
    # is_date
    # --------------------------------
    # Verifica se é uma data
    # Retorna boolean
    #
    @staticmethod
    def is_date(value):
        status = False
        if (type(value) == dict) and ("$date" in value.keys()):
            status = (len(value["$date"]) == 24)
        return status

    # --------------------------------
    # is_decimal
    # --------------------------------
    # Verifica se é decimal
    # Retorna boolean
    #
    @staticmethod
    def is_decimal(value):
        status = False
        if (type(value) == dict) and ("$numberDecimal" in value.keys()) and re.match("""^-?[0-9]+\.?[0-9]*$""", value["$numberDecimal"]):
            status = (len(value["$numberDecimal"]) > 0) and (len(value["$numberDecimal"]) <= 34)
        return status

    # --------------------------------
    # is_number
    # --------------------------------
    # Verifica se é número
    # Retorna boolean
    #
    @staticmethod
    def is_number(value):
        return (type(value) == Int64) and (value >= -9223372036854775808) and (value <= 9223372036854775807)

    # --------------------------------
    # is_object
    # --------------------------------
    # Verifica se é objeto
    # Retorna boolean
    # TODO:: Implementar validação recursiva para valores numéricos
    #
    @staticmethod
    def is_object(value):
        return (type(value) == dict) and ("error" not in value.keys())

    # --------------------------------
    # is_object_id
    # --------------------------------
    # Verifica se é objectId
    # Retorna boolean
    #
    @staticmethod
    def is_object_id(value):
        return type(value) == ObjectId

    # --------------------------------
    # is_string
    # --------------------------------
    # Verifica se é string
    # Retorna boolean
    #
    @staticmethod
    def is_string(value):
        return (type(value) == unicode) or (type(value) == str)

    # --------------------------------
    # is_timestamp
    # --------------------------------
    # Verifica se é timestamp
    # Retorna boolean
    #
    @staticmethod
    def is_timestamp(value):
        return type(value) == Timestamp

    # --------------------------------
    # to_array
    # --------------------------------
    # Converte em array todas as opções válidas recebidas
    # Retorna o valor original, caso o tipo não seja possível converter
    #
    @staticmethod
    def to_array(value):
        # Verifica se a string está no formato array
        if re.match(r"\[([\s\S]*)\]\Z", value, re.IGNORECASE):
            try:
                value = value.replace("'", '"')
                value = loads(value)
            except Exception as inst:
                value = {"error": str(inst.args)}
        # Caso contrário tentar converter para array
        elif re.match("""^([^,'";]+,[ ]?)+([^,'";]+)$""", value, re.IGNORECASE):
            value = value.split(",")
        return value

    # --------------------------------
    # to_boolean
    # --------------------------------
    # Converte em boolean todas as opções válidas recebidas
    # Retorna o valor original, caso o tipo não seja possível converter
    #
    @staticmethod
    def to_boolean(value):
        if value.lower() in ["true", "1"]:
            value = True
        elif value.lower() in ["false", "0"]:
            value = False
        return value

    # --------------------------------
    # to_date
    # --------------------------------
    # Converte em data todas as opções válidas recebidas
    # Retorna o valor original, caso o tipo não seja possível converter
    #
    @staticmethod
    def to_date(value):
        regex = r"([0-9]{4})-([0-1][0-9])-([0-3][0-9])[T ]?([0-2][0-9])?:?([0-5][0-9])?:?([0-5][0-9])?\.?([0-9]{0,9})?Z?\Z"
        match = re.match(regex, value, re.IGNORECASE)
        if match:
            date_parts = ["0000", "00", "00", "00", "00", "00", "000"]
            index = 0
            for item in match.groups():
                if item is not None:
                    date_parts[index] = item
                    index = index + 1

            if DataTypes.to_date_is_valid(date_parts):
                value = DataTypes.to_date_format(date_parts)
        return value

    # --------------------------------
    # to_date_is_valid
    # --------------------------------
    # Verifica se uma data é válida
    # Retorna boleano
    # @private
    #
    @staticmethod
    def to_date_is_valid(parts):
        return (int(parts[1]) <= 12) and (int(parts[2]) <= 31) and (int(parts[3]) <= 23) and (int(parts[4]) <= 59) and (int(parts[5]) <= 59)

    # --------------------------------
    # to_date_format
    # --------------------------------
    # Converte valor recebido para o formato data
    # Retorna objeto
    # @private
    #
    @staticmethod
    def to_date_format(parts):
        date_str = str(parts[0] + "-" + parts[1] + "-" + parts[2] + "T" + parts[3] + ":" + parts[4] + ":" + parts[5])
        try:
            date_str = datetime.datetime(*time.strptime(date_str, "%Y-%m-%dT%H:%M:%S")[:6])
            value = {"$date": date_str.strftime("%Y-%m-%dT%H:%M:%S") + "." + str(parts[6][:3]) + "Z"}
        except Exception as inst:
            value = {"error": str(inst.args)}

        return value

    # --------------------------------
    # to_decimal
    # --------------------------------
    # Converte em decimal todas as opções válidas recebidas
    # Retorna o valor original, caso o tipo não seja possível converter
    #
    @staticmethod
    def to_decimal(value):
        # Remove todos os caracteres nao numericos
        value = re.sub(r'[^0-9.-]+', '', value)
        if len(value) > 0:
            value = {"$numberDecimal": DataTypes.to_str(value)}
        return value

    # --------------------------------
    # to_number
    # --------------------------------
    # Converte o valor recebido para o formato número
    # Retorna inteiro ou o valor original (somente números) se não puder converter
    #
    @staticmethod
    def to_number(value):
        # Remove todos os caracteres não numéricos
        value = re.sub(r'[^0-9-]+', '', value)
        if len(value) > 0:
            value = Int64(long(value))
        return value

    # --------------------------------
    # to_object
    # --------------------------------
    # Converte o valor recebido para o formato objeto
    # Retorna objeto ou o valor original caso não consiga converter
    #
    @staticmethod
    def to_object(value):
        try:
            value = value.replace("'", '"')
            value = loads(value)
        except Exception as inst:
            value = {"error": str(inst.args)}
        return value

    # --------------------------------
    # to_object_id
    # --------------------------------
    # Converte o valor recebido para o formato ObjectId
    # Retorna ObjectId ou o valor original caso não consiga converter
    #
    @staticmethod
    def to_object_id(value):
        match = re.match("^([0-9a-f]{24})$", value, re.IGNORECASE)
        if match:
            value = match.group(1)
            if len(value) == 24:
                try:
                    value = ObjectId(value)
                except Exception as inst:
                    value = {"error": str(inst.args)}
        return value

    # --------------------------------
    # to_str
    # --------------------------------
    # Converte o valor recebido para o formato String
    # Retorna string
    #
    @staticmethod
    def to_str(value):
        try:
            if type(value) == unicode:
                value = unicodedata.normalize('NFKD', value).encode('utf-8', 'ignore')
            else:
                value = str(value)
        except Exception as inst:
            value = {"error": str(inst.args)}
        return value

    # --------------------------------
    # to_timestamp
    # --------------------------------
    # Converte o valor recebido para o formato Timestamp
    # Retorna Timestamp ou o valor original caso não consiga converter
    # TODO:: Implementar outras opções Default de timestamp (s1500, m30, h5, d15, m13, y23)
    #
    @staticmethod
    def to_timestamp(value):
        if value == 'now':
            value = Timestamp(datetime.datetime.utcnow(), 0)
        else:
            try:
                value = re.sub(r'[^0-9]+', '', value)
                value = Timestamp(int(value), 0)
            except Exception as inst:
                value = {"error": str(inst.args)}
        return value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datarest import DataRest
from database import DataBase
from datatypes import DataTypes
from status import Status


class Service(object):
    def __init__(self, message, service_id, name, schemes, actions, custom_service=None):
        self.actions = actions
        self.id = service_id
        self.name = name
        self.schemes = schemes
        self.status = Status()

        self.__data_types = DataTypes()
        self.__data_base = DataBase()
        self.__data_rest = DataRest(message, custom_service, self.status)
        self.__max_limit = 100

    # --------------------------------
    # action
    # --------------------------------
    def action(self):
        source = self.__data_rest.source()
        method = self.__data_rest.method()

        # Verifica se o endereço da api existe
        if source in self.actions[method].keys():
            # Recupera o objeto de configuração da api
            service_api = self.actions[method][source]
            action = service_api["action"]
            action_collection = action["collection"]
            action_db = action["database_action"]
            action_scheme = action["scheme"]
            action_type = action["result_type"]

            self.__data_rest.result_type(action_type)

            if hasattr(self, action_db) and self.__data_rest.validate_service(self.name, self.id):
                method = getattr(self, action_db)
                if action_db == "insert":
                    method(action_collection, self.__get_scheme(action_scheme, self.schemes))
                elif action_db == "remove":
                    method(action_collection)
                elif action_db == "update":
                    method(action_collection, self.__get_scheme(action_scheme, self.schemes), self.schemes)
                else:
                    method(action_collection, self.__get_scheme(self.__get_scheme(action_scheme, self.schemes), self.schemes),
                           self.get_max_limit(action))
            else:
                self.status.error("INVALID_ACTION", None, [source, self.name, self.__data_types.to_str(service_api["action"]), method])

        else:
            self.status.error("INVALID_ENDPOINT", None, [source, self.name, method])

    # --------------------------------
    # find
    # --------------------------------
    def find(self, collection, list_scheme, max_limit=None):
        # Define o tipo de ação que será executada no servidor de base de dados
        self.__data_rest.action_db("find")

        # Recupera o identificador da coleção
        self.__data_rest.collection(collection)

        # Prepara a lista de campos válidos que poderão ser recuperados na base de dados
        # Recebe a lista de erros em campos definido pelo desenvolvedor de microserviço
        fields, dev_fields_errors = self.__data_base.fields_find(list_scheme)

        # Caso o campo esteja em conflito no modo desenvolvedor de microserviço, registra mensagem de error
        for item_field_name in dev_fields_errors:
            self.status.error("FIELD_CONFLICT", None, [item_field_name])

        # Registra campos que devem ser retornados
        self.__data_rest.fields(fields)
        self.__data_rest.limit(max_limit)

    # --------------------------------
    # get_max_limit
    # --------------------------------
    @staticmethod
    def get_max_limit(action):
        if "max_limit" in action:
            return action["max_limit"]
        return None

    # --------------------------------
    # insert
    # --------------------------------
    # Prepara documento para ser inserido base de dados
    def insert(self, collection, list_scheme):
        list_insert_data = []

        # Define o tipo de ação que será executada no servidor de base de dados
        self.__data_rest.action_db("insert")

        # Recupera o identificador da coleção
        self.__data_rest.collection(collection)

        # Prepara a lista de campos válidos que poderão ser inseridos na base de dados
        # Recebe a lista de erros em campos recebidos no body
        # Recebe a lista de erros em campos definido pelo desenvolvedor de microserviço
        list_scheme_clean, body_field_errors, dev_fields_errors = self.__data_base.fields_insert(self.__data_rest.body, list_scheme)

        # Processa todos os campos registrados no schema para validar os valores e formatar
        for item_scheme_name in list_scheme_clean:
            # Recupera o esquema para validar e formatar
            item_scheme = list_scheme_clean[item_scheme_name]

            # Recupera o valor que deverá ser validado e formatado
            field_value = item_scheme["value"]

            # Valida o campo e valor
            # Recebe o nome do campo que será inserido na base de dados e depois retornado
            # Recebe o valor que deverá ser insrido na base de dados
            field_name, field_value = self.__data_rest.field_insert(item_scheme_name, item_scheme, field_value, "custom_insert")

            # Verifica se o campo não está com erro de validação
            if field_value != '__error__' and field_value != '__ERROR__':
                # Registra o campo na lista de dados que deverá ser enviada para o servidor de base de dados
                if field_name != "id":
                    list_insert_data.append({"field": field_name, "value": field_value})

        # Prepara s lista de campos que será enviada para o servidor de base de dados
        # Prepara a lista de campos que devrá ser retornada do servidor depois de salvar na base de dados
        list_insert_data = self.__prepare_insert_data_fields(list_insert_data)

        # Registra os dados do documento
        self.__data_rest.data(list_insert_data)

        # Registra as datas de criação e alteração do documento
        self.__data_rest.created_time()
        self.__data_rest.changed_time()

        # Caso o campo nao esteja registrado no scheme, registra mensagem de error
        for item_field_name in body_field_errors:
            self.status.error("DOES_NOT_EXIST_FIELD", None, [item_field_name])

        # Caso o campo esteja em conflito no modo desenvolvedor de microserviço, registra mensagem de error
        for item_field_name in dev_fields_errors:
            self.status.error("FIELD_CONFLICT", None, [item_field_name])

        # Verificar se tem algo para gravar no banco, caso contrário, registra mensagem de error
        if len(list_insert_data) <= 0:
            self.status.error("NO_DATA_TO_INSERT", None, [])

    # --------------------------------
    # is_valid_scheme
    # --------------------------------
    @staticmethod
    def is_valid_scheme(scheme):
        if type(scheme) == dict:
            list_scheme = ["$currentDate", "$inc", "$min", "$max", "$mul", "$rename", "$set", "$setOnInsert", "$unset"]
            for key in scheme.keys():
                if key in list_scheme:
                    return True
        return False

    # --------------------------------
    # prepare_update_data
    # --------------------------------
    # Prepara lista de campos que podem ser atualizados no documento
    @staticmethod
    def prepare_update_data(list_data):
        result = {}
        for item in list_data:
            # Registra o campo e valor na lista de campos válida do documento
            result[(item["field"])] = item["value"]
        return result

    # --------------------------------
    # ready
    # --------------------------------
    # TODO:: Implementar melhorias na verificação de disponibilidade do serviço para serguir adiante, chegando todos os tipos que serão enviados
    def ready(self):
        return self.status.ready()

    # --------------------------------
    # remove
    # --------------------------------
    def remove(self, collection):
        # Define o tipo de ação que será executada no servidor de base de dados
        self.__data_rest.action_db("remove")

        # Recupera o identificador da coleção
        self.__data_rest.collection(collection)

    # --------------------------------
    # send_database
    # --------------------------------
    def send_database(self):
        return self.__data_rest.send_database()

    # --------------------------------
    # send_error
    # --------------------------------
    def send_error(self):
        return self.__data_rest.send_error()

    # --------------------------------
    # update
    # --------------------------------
    def update(self, collection, scheme, list_scheme):
        # Validar configuração de schema para update, erro developer
        if self.is_valid_scheme(scheme):
            # Define o tipo de ação que será executada no servidor de base de dados
            self.__data_rest.action_db("update")

            # Recupera o identificador da coleção
            self.__data_rest.collection(collection)

            update_data = {}
            fields = ["_id"]
            for item_scheme in scheme:
                obj_item_scheme = self.__get_scheme(scheme[item_scheme], list_scheme)
                obj_item_scheme = self.update_config(obj_item_scheme)

                if len(obj_item_scheme.keys()) > 0:
                    fields = fields + obj_item_scheme.keys()
                    update_data[item_scheme] = obj_item_scheme

            # Registra os dados do documento
            self.__data_rest.data(update_data)

            # Registra campos que devem ser retornados
            self.__data_rest.fields(fields)

            # Registra as datas de criação e alteração do documento
            self.__data_rest.changed_time()

            # Verificar se tem algo para gravar no banco
            if len(update_data.keys()) <= 0:
                self.status.error("NO_DATA_TO_UPDATE", None, [])
        else:
            self.status.error("INVALID_SCHEME_UPDATE", None, [])

    # --------------------------------
    # update_config
    # --------------------------------
    # TODO:: implementar checksum de update
    def update_config(self, list_scheme):

        list_update_data = []

        # Prepara a lista de campos válidos que poderão ser inseridos na base de dados
        # Recebe a lista de erros em campos recebidos no body
        # Recebe a lista de erros em campos definido pelo desenvolvedor de microserviço
        list_scheme_clean, body_field_errors, dev_fields_errors = self.__data_base.fields_update(self.__data_rest.body, list_scheme)

        # Processa todos os campos registrados no schema para validar os valores e formatar
        for item_scheme_name in list_scheme_clean:
            # Recupera o equema para validar e formatar
            item_scheme = list_scheme_clean[item_scheme_name]

            # Recupera o valor que deverá ser validado e formatado
            field_value = item_scheme["value"]

            # Valida o campo e valor
            # Recebe o nome do campo que será inserido na base de dados e depois retornado
            # Recebe o valor que deverá ser inserido na base de dados
            field_name, field_value = self.__data_rest.field_update(item_scheme_name, item_scheme, field_value, "custom_update")

            # Verifica se o campo não está com erro de validação
            if field_value != '__error__' and field_value != '__ERROR__':
                # Registra o campo na lista de dados que deverá ser enviada para o servidor de base de dados
                if field_name != "id":
                    list_update_data.append({"field": field_name, "value": field_value})

        # Prepara a lista de campos que será enviada para o servidor de base de dados
        # Prepara a lista de campos que devrá ser retornada do servidor depois de salvar na base de dados
        list_update_data = self.prepare_update_data(list_update_data)

        # Caso o campo nao esteja registrado no scheme, registra mensagem de error
        for item_field_name in body_field_errors:
            self.status.error("DOES_NOT_EXIST_FIELD", None, [item_field_name])

        # Caso o campo esteja em conflito no modo desenvolvedor de microserviço, registra mensagem de error
        for item_field_name in dev_fields_errors:
            self.status.error("FIELD_CONFLICT", None, [item_field_name])

        # Verificar se tem algo para gravar no banco, caso contrário, registra mensagem de error
        if len(list_update_data) <= 0:
            self.status.error("NO_DATA_TO_UPDATE", None, [])

        return list_update_data

    # --------------------------------
    # __get_scheme
    # --------------------------------
    def __get_scheme(self, scheme, list_schemes):
        result = {}
        for field_name in scheme:
            if field_name in list_schemes.keys():
                result[field_name] = list_schemes[field_name]
            else:
                self.status.error("DOES_NOT_EXIST_FIELD", None, [field_name])
        return result

    # --------------------------------
    # __insert_data_join
    # --------------------------------
    # Função recursiva utilizada para tratar conflito de elementos filho no momento de gravar na base de dados
    #
    # data - Objeto contento dados existentes do mesmo grupo ou um objeto vazio para receber novos elementos
    # value - Dados que deverão ser inseridos no objeto
    def __insert_data_join(self, data, value):

        # Pega cada elemento do objeto valor
        for item in value:
            # Verifica se valor é um objeto
            # Verifica se o elemento filho valor é um objeto
            # Verifica se o elemento filho existe no objeto de dados
            if type(value) == dict and type(value[item]) == dict and type(data) == dict and item in data.keys():
                # Adiciona novo elemento no objeto de dados com o resultado da interação nos objetos filhos do objeto valor
                data[item] = self.__insert_data_join(data[item], value[item])

            # Verifica se valor é um objeto
            # Verifica se o elemento filho valor é um objeto
            elif type(value) == dict and type(data) == dict:
                # Adiciona o objeto valor como novo elemento no objeto de dados
                data[item] = value[item]

            # Verifica se o objeto dados é um objeto
            elif type(data) == dict:
                # Adiciona o valor como novo elemento no objeto de dados
                data[item] = value
        return data

    # --------------------------------
    # __prepare_insert_data_fields
    # --------------------------------
    # Prepara lista de campos que podem ser incluidos no documento
    # Prepara e registra a lista de campos que deverá ser retornada do servidor após a inclusão do documento na base de dados
    def __prepare_insert_data_fields(self, list_data):
        result = {}
        fields = {}

        for item in list_data:
            # Verifica se o nome do campo existe na lista que será incluida no documento
            if (item["field"]) in result.keys():
                # Caso não esteja na lista então o campo é incluido
                # Prepara objetos filhos no padrão permitido paraga registrar na base de dados
                result[(item["field"])] = self.__insert_data_join(result[(item["field"])], item["value"])
            else:
                # Registra o campo e valor na lista de campos válida do documento
                result[(item["field"])] = item["value"]

            # Adiciona o campo na lista de campos que devem ser retornados do servidor
            fields[(item["field"])] = True

        # Registra campos que devem ser retornados
        self.__data_rest.fields(fields)

        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import bson
import time
import datetime
from pymongo import MongoClient
from datarest_db import DataRestDB
from datatypes import DataTypes
from status import Status


# TODO:: Implementar multi-idiomas classe ServiceDB
# TODO:: Implementar try catch para a criação do banco de dados mongo db
# TODO:: Refatorar classe ServiceDB
# TODO:: Implementar identificação de owner_id, last_owner_id, api_key, app_id, group
class ServiceDB:
    def __init__(self, message, mongo_ip, mongodb_port):

        # Conecta ao banco de dados
        self.mongo_server = MongoClient(mongo_ip, mongodb_port)
        self.status = Status()
        self.__data_types = DataTypes()
        self.__data_rest_db = DataRestDB(message, self.status)

        self.__action()

    # --------------------------------
    # find
    # --------------------------------
    # Realiza consulta na base de dados buscando um ou muitos documentos
    # TODO:: Implementar multi-idiomas classe ServiceDB::find
    # TODO:: Preparar filtro para ser retornado ao cliente
    # TODO:: Preparar fields ser retornado ao cliente
    # TODO:: Preparar conversão de dados ser retornado ao cliente (date, object_id, decimal)
    # TODO:: Implementar links de paginador
    def find(self):
        # Recupera a coleção principal
        collection = self.__collection()

        # Recupera configurações
        db_fields = self.__data_rest_db.fields()
        db_filter = self.__data_rest_db.filter()
        db_limit = self.__data_rest_db.limit()
        db_offset = self.__data_rest_db.offset()
        db_sort = self.__data_rest_db.sort()

        # Consulta o banco de dados
        find_result = collection.find(db_filter, db_fields).skip(db_offset).sort(db_sort.items())
        finded_count = find_result.count()

        # Obtém o resultado da consulta
        find_result_limit = list(find_result.limit(db_limit))

        # Prepara os dados dos documentos recuperados $date, $timestamp ...
        find_result_limit, filtered_count = self.__prepare_result(find_result_limit)

        data_finded = {}
        # Prepara os resultados para serem retornados ao cliente
        if "item" == self.__data_rest_db.result_type():
            if filtered_count == 1:
                data_finded = find_result_limit[0]
            else:
                self.status.warn("WARN_FIND_ITEM", None, [])

        elif "list" == self.__data_rest_db.result_type():
            data_finded = find_result_limit

        if finded_count == 0:
            message = "Nenhum documento encontrado."
        elif finded_count == 1:
            message = "1 documento encontrado."
        else:
            message = self.__data_types.to_str(filtered_count) + " documentos encontrados."

        self.__data_rest_db.result({
            "success": True,
            "message": message,
            "data_finded": data_finded,
            "count": {
                "finded": finded_count,
                "filtered": filtered_count
            },
            "query": {
                "limit": db_limit,
                "offset": db_offset,
                "fields": db_fields,
                "filter": db_filter,
                "sort": db_sort
            },
            "links": {
                "previous": None,
                "next": "https://api.predicthq.com/v1/endpoint/?offset=10&limit=10"
            }
        })

    # --------------------------------
    # insert
    # --------------------------------
    # Insere um ou muitos documentos na base de dados
    # TODO:: Implementar multi-idiomas classe ServiceDB::insert
    def insert(self):
        # Recupera a coleção principal
        collection = self.__collection()

        # Recupera dados e configurações
        db_data = self.__data_rest_db.data()
        db_fields = self.__data_rest_db.fields()

        # Verifica o tipo de dados
        if type(db_data) == dict and len(db_data.keys()) > 0:
            db_data = [db_data]

        data_inserted = {}
        # Verifica se tem dados para inseretir na base de dados
        if len(db_data) > 0:
            # Insere documentos na base de dados e recupera a lista dos identificadores dos documentos inseridos
            insert_many_result = collection.insert_many(db_data).inserted_ids

            # Verifica se algum documento foi inserido
            if len(insert_many_result) > 0:
                # Prepara filtro para recuperar lista de documentos inseridos
                db_filter = {"_id": {"$in": insert_many_result}}

                # Realiza consulta na base de dados e recupera a lista de documentos inseridos
                find_result = collection.find(db_filter, db_fields)

                # Prepara os dados dos documentos recuperados $date, $timestamp ...
                find_result, inserted_count = self.__prepare_result(find_result)

                # Prepara os resultados para serem retornados ao cliente
                if "item" == self.__data_rest_db.result_type():
                    if inserted_count == 1:
                        data_inserted = find_result[0]
                    elif inserted_count > 1:
                        data_inserted = find_result[0]
                        self.status.warn("WARN_INSERT_ITEM", None, [])

                elif "list" == self.__data_rest_db.result_type():
                    data_inserted = find_result

                if inserted_count == 1:
                    message = "1 documento foi inserido com sucesso."
                else:
                    message = self.__data_types.to_str(inserted_count) + " documentos inseridos com sucesso."

                # Prepara configurações que serão retornadas para o cliente
                self.__data_rest_db.result({
                    "success": True,
                    "message": message,
                    "data_inserted": data_inserted,
                    "count": {
                        "inserted": inserted_count
                    },
                    "query": {
                        "fields": db_fields
                    }
                })
        else:
            # Em case de erros por fata de documentos para inserir na base de dados ou qualquer outra falha no momento da inserção
            # registra mensagem de erro e configura a flha na operação
            self.status.error("NO_DATA_TO_INSERT", None, [])

            # Prepara os resultados para serem retornados ao cliente
            if "item" == self.__data_rest_db.result_type():
                data_inserted = {}
            else:
                data_inserted = []

            message = "Nenhum documento foi inserido."

            # Registra resultado que será retornado para o servidor rest
            self.__data_rest_db.result({
                "success": False,
                "message": message,
                "data_inserted": data_inserted,
                "count": {
                    "inserted": 0
                },
                "query": {
                    "fields": db_fields
                }
            })

    # --------------------------------
    # prepare_result_update
    # --------------------------------
    # TODO:: Refatorar classe ServiceDB:: prepare_result_update
    @staticmethod
    def prepare_result_update(data):
        list_scheme = ["$currentDate", "$inc", "$min", "$max", "$mul", "$rename", "$set", "$setOnInsert", "$unset"]
        data_update = {}
        for key in data:
            if key in list_scheme:
                for item in data[key]:
                    data_update[item] = data[key][item]

        return data_update

    # --------------------------------
    # ready
    # --------------------------------
    def ready(self):
        return self.status.ready()

    # --------------------------------
    # remove
    # --------------------------------
    # Remove um ou muitos documentos na base de dados
    #
    # TODO:: Implementar solução final para tratar _ID
    # TODO:: Resolver problemas exclusão de um em usando $in, item/lista de ids em filtros in {'_id': {'$in': ObjectId('5aad1be48356691750d406c0')}}
    # TODO:: Implementar multi-idiomas classe ServiceDB::remove
    def remove(self):
        # Recupera a coleção principal
        collection = self.__collection()

        # Recupera dados e configurações
        db_filter = self.__data_rest_db.filter()

        if "id" in db_filter.keys():
            db_filter["_id"] = bson.ObjectId(db_filter["id"]["$eq"])
            del db_filter["id"]

        # Realiza busca na colecção para recuperar documentos que serão atualizados
        find_result = collection.find(db_filter)
        find_result_count = find_result.count()

        # Verifica se existe documentos para serem atualizados
        deleted_count = 0
        deleted_ids = []
        if find_result_count > 0:
            # Recupera a coleção trash
            collection_trash = self.__collection_trash()

            # Grava os documentos localizados na coleção trash
            insert_data_trash, deleted_ids = self.__prepare_data_trash(find_result)
            insert_many_result = collection_trash.insert_many(insert_data_trash).inserted_ids

            # Verifica se foi inserido documento na coleção trash
            if len(insert_many_result) > 0:
                # Atualiza os documentos da coleção principal
                delete_result = collection.delete_many(db_filter)
                deleted_count = delete_result.deleted_count

                # Verifica se os documentos não foram atualizados
                if deleted_count == 0:
                    # Remove itens inseridos na tabela trash
                    collection_trash.delete_many({"_id": {"$in": insert_many_result}})

        if deleted_count == 0:
            message = "Nenhum documento foi removido."
        elif deleted_count == 1:
            message = "1 documento removido com sucesso."
        else:
            message = self.__data_types.to_str(deleted_count) + " documentos removidos com sucesso."

        # Prepara configurações que serão retornadas para o cliente
        self.__data_rest_db.result({
            "success": True,
            "message": message,
            "deleted_ids": deleted_ids,
            "count": {
                "deleted": deleted_count
            },
            "query": {
                "filter": db_filter
            },
            "info": {}
        })

    # --------------------------------
    # send
    # --------------------------------
    def send(self):
        return self.__data_rest_db.send()

    # --------------------------------
    # update
    # --------------------------------
    # TODO:: Implementar multi-idiomas classe ServiceDB::update
    def update(self):
        # Recupera a coleção principal
        collection = self.__collection()

        # Recupera dados e configurações
        db_data = self.__data_rest_db.data()
        db_fields = self.__data_rest_db.fields()
        db_filter = self.__data_rest_db.filter()

        # Realiza busca na colecção para recuperar documentos que serão atualizados
        find_result = collection.find(db_filter, db_fields)
        find_result_count = find_result.count()

        # Verifica se existe documentos para serem atualizados
        update_count = 0
        if find_result_count > 0:
            # Recupera a coleção changes
            collection_changes = self.__collection_changes()

            # Grava os documentos localizados na coleção trash
            insert_result = collection_changes.insert(self.__prepare_data_changes(db_data, find_result), check_keys=False)

            # Verifica se foi inserido documento na coleção _changes
            if insert_result:
                # Atualiza os documentos da coleção principal
                update_result = collection.update_many(db_filter, db_data, upsert=True)
                update_count = update_result.modified_count

                # Verifica se os documentos não foram atualizados
                if update_count == 0:
                    # Remove itens inseridos na tabela change
                    collection_changes.delete_one({"_id": insert_result})

        # Prepara os dados dos documentos recuperados $date, $timestamp ...
        update_result = self.prepare_result_update(db_data)

        data_updated = {}
        # Prepara os resultados para serem retornados ao cliente
        if "item" == self.__data_rest_db.result_type():
            if update_count == 1:
                data_updated = update_result
            elif update_count > 1:
                data_updated = update_result[0]

        elif "list" == self.__data_rest_db.result_type():
            data_updated = update_result
            if type(update_result) == dict:
                data_updated = [update_result]

        if update_count == 0:
            message = "Nenhum documento foi atualizado."
        elif update_count == 1:
            message = "1 documento atualizado com sucesso."
        else:
            message = self.__data_types.to_str(update_count) + " documentos atualizados com sucesso."

        # Prepara configurações que serão retornadas para o cliente
        self.__data_rest_db.result({
            "success": True,
            "message": message,
            "data_updated": data_updated,
            "count": {
                "modified": update_count
            },
            "query": {
                "fields": db_fields,
                "filter": db_filter
            }
        })

    # --------------------------------
    # __action
    # --------------------------------
    def __action(self):
        action = self.__data_rest_db.action()
        if hasattr(self, action):
            method = getattr(self, action)
            method()
        else:
            self.status.error("INVALID_ACTION_DB", None, [action])

    # --------------------------------
    # __collection
    # --------------------------------
    # Recupera instancia da coleção que receberá interação na base de dados
    # TODO:: Verificar se o banco de dados esta instanciado antes de tentar criar a coleção
    def __collection(self):
        # Recupera o banco de dados mongo db
        database = self.mongo_server[self.__data_rest_db.database()]
        # Recupera/Cria a coleção mongo db
        return database[self.__data_rest_db.collection()]

    # --------------------------------
    # __collection_changes
    # --------------------------------
    def __collection_changes(self):
        # Recupera o banco de dados mongo db
        database = self.mongo_server[self.__data_rest_db.database()]
        # Recupera/Cria a coleção mongo db
        return database["_changes"]

    # --------------------------------
    # __collection_trash
    # --------------------------------
    def __collection_trash(self):
        # Recupera o banco de dados mongo db
        database = self.mongo_server[self.__data_rest_db.database()]
        # Recupera/Cria a coleção mongo db
        return database["_trash"]

    # --------------------------------
    # __prepare_data_changes
    # --------------------------------
    # TODO:: Refatorar classe ServiceDB:: __prepare_data_changes
    def __prepare_data_changes(self, data_update, data_active):
        data_active = list(data_active)

        def get(d, keys):
            if "." in keys:
                k, rest = keys.split(".", 1)
                if k in d.keys():
                    return get(d[k], rest)
                return "__undefined__"
            else:
                if keys in d.keys():
                    return d[keys]
                return "__undefined__"

        documents = {}
        # gupos de interação $set, $inc ...
        for item in data_update:

            # itens de cada grupo
            for data_item in data_update[item]:
                # itens dos documento da base de dados
                for item_find in data_active:
                    active_id = item_find["_id"]

                    if active_id not in documents.keys():
                        documents[active_id] = {
                            "change_data_active": [],
                            "change_data_update": []
                        }

                    value_update = data_update[item][data_item]
                    value_active = get(item_find, data_item)
                    if value_active != value_update:
                        if value_active != "__undefined__":
                            active = {}
                            active[data_item] = value_active
                            (documents[active_id]["change_data_active"]).append(active)

                        update = {}
                        update[data_item] = value_update
                        (documents[active_id]["change_data_update"]).append(update)

        data = {}

        for id_doc in documents:
            doc_active = {}
            list_data_active = documents[id_doc]["change_data_active"]
            for item_data_active in list_data_active:
                key = item_data_active.keys()
                doc_active[(key[0])] = item_data_active[(key[0])]
            if len(doc_active.keys()) > 0:
                if self.__data_types.to_str(id_doc) not in data:
                    data[self.__data_types.to_str(id_doc)] = {}
                data[self.__data_types.to_str(id_doc)]["active"] = doc_active

            doc_update = {}
            list_data_update = documents[id_doc]["change_data_update"]
            for item_data_update in list_data_update:
                key = item_data_update.keys()
                doc_update[(key[0])] = item_data_update[(key[0])]
            if len(doc_update.keys()) > 0:
                if self.__data_types.to_str(id_doc) not in data:
                    data[self.__data_types.to_str(id_doc)] = {}
                data[self.__data_types.to_str(id_doc)]["update"] = doc_update

        insert_data_update = {
            "filter": {},
            "data": data,
            "created_time": self.__data_types.to_str(datetime.datetime.utcnow()),
            "owner_id": "5aa73a758356693ffc73499c",
            "collection": self.__data_rest_db.collection()
        }

        return insert_data_update, data

    # --------------------------------
    # __prepare_data_trash
    # --------------------------------
    # TODO:: Refatorar classe ServiceDB:: __prepare_data_trash
    def __prepare_data_trash(self, data):
        # Retirar list de ids que foram removidos da base de dados
        deleted_ids = []
        insert_data_trash = []

        for item in data:
            deleted_ids.append(self.__data_types.to_str(item["_id"]))
            insert_data_trash.append({
                "data": item,
                "created_time": self.__data_types.to_str(datetime.datetime.utcnow()),
                "owner_id": "5aa73a758356693ffc73499c",
                "collection": self.__data_rest_db.collection()
            })
        return insert_data_trash, deleted_ids

    # --------------------------------
    # __prepare_result
    # --------------------------------
    # TODO:: Refatorar classe ServiceDB:: __prepare_result
    def __prepare_result(self, data):
        def action(item_result):
            result = {}
            for item_field in item_result:
                # Converte ObjectId para string
                if type(item_result[item_field]) == bson.ObjectId:
                    if "_id" == item_field:
                        result['id'] = self.__data_types.to_str(item_result[item_field])
                    else:
                        result[item_field] = self.__data_types.to_str(item_result[item_field])

                # datetime.datetime
                # Converte datetime
                elif type(item_result[item_field]) == datetime.datetime:
                    # format = "%a %b %d %H:%M:%S %Y"
                    result[item_field] = (item_result[item_field]).isoformat()

                # Converte Decimal128 para Float
                elif type(item_result[item_field]) == bson.decimal128.Decimal128:
                    result[item_field] = float(self.__data_types.to_str(item_result[item_field]))

                # TODO:: Estudar datas do Python:  https://docs.python.org/2/library/datetime.html
                # TODO:: Extrair timestamp no horário correto
                # Converte Timestamp para number
                elif type(item_result[item_field]) == bson.timestamp.Timestamp:
                    result[item_field] = time.mktime((item_result[item_field]).as_datetime().timetuple())
                    # datetime.timestamp() === 1234567890   time.time()
                    # datetime.isoformat(sep='T', timespec='auto') === YYYY-MM-DDTHH:MM:SS.mmmmmm
                else:
                    result[item_field] = item_result[item_field]

            return result

        final_result = list(map(action, data))
        return final_result, len(final_result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# TODO:: Refatorar completamente a classe Status
# TODO:: Implementar multi-idiomas classe Status
# TODO:: Remover de dentro da classe Status todas as mensagens
class Status:
    def __init__(self):
        self.__has_error = False
        self.__has_info = False
        self.__has_log = False
        self.__has_warn = False

        self.__list_errors = []

        self.error_messages = {
            "TYPE_INVALID_ARRAY": {
                "code": 4505845,
                "message": "O valor do campo '{0}' não pode ser '{1}', vazio ou null, utilize um valor do tipo array [\"a\", \"b\", 1, 2]",
                "message_except": "ERRO ARRAY no campo {0}, {1}"
            },
            "TYPE_INVALID_BOOLEAN": {
                "code": 5454575,
                "message": "O valor do campo '{0}' não pode ser '{1}', vazio ou null, utilize um valor entre true | false | 0 | 1",
                "message_except": "ERRO BOOLEAN no campo {0}, {1}"
            },
            "TYPE_INVALID_DATE": {
                "code": 5454587,
                "message": "O valor do campo '{0}' não pode ser '{1}', vazio ou null, utilize os formatos de data AAAA-MM-DD ou AAAA-MM-DDTHH:MM:SS.mmm",
                "message_except": "ERRO DATE no campo {0}, {1}"
            },
            "TYPE_INVALID_DECIMAL": {
                "code": 8545255,
                "message": "O valor do campo '{0}' não pode ser '{1}', vazio ou null, utilize um valor entre -x até x.",
                "message_except": "ERRO DECIMAL no campo {0}, {1}"
            },
            "TYPE_INVALID_ENUM": {
                "code": 4505845,
                "message": "O valor do campo '{0}' não pode ser '{1}', vazio ou null, utilize uma das opções de valor : {2}",
                "message_except": "ERRO ENUM no campo {0}, {1}"
            },
            "TYPE_INVALID_NUMBER": {
                "code": 8545255,
                "message": "O valor do campo '{0}' não pode ser '{1}', vazio ou null, utilize um valor entre -9223372036854775808 até 9223372036854775807.",
                "message_except": "ERRO NUMBER no campo {0}, {1}"
            },
            "TYPE_INVALID_OBJECT": {
                "code": 545465454,
                "message": "O valor do campo '{0}' não pode ser '{1}', vazio ou null, utilize um valor do tipo object json.",
                "message_except": "ERRO OBJECT no campo {0}, {1}"
            },
            "TYPE_INVALID_OBJECT_ID": {
                "code": 6565482,
                "message": "O valor do campo '{0}' não pode ser '{1}', vazio ou null, utilize um valor do tipo object_id, exemplo: '5abcd65b8356694714f893e9'.",
                "message_except": "ERRO OBJECT_ID no campo {0}, {1}"
            },
            "TYPE_INVALID_TIMESTAMP": {
                "code": 32456454,
                "message": "O valor do campo '{0}' não pode ser '{1}', vazio ou null, utilize um valor do tipo timestamp, exemplo: '1522343386'.",
                "message_except": "ERRO TIMESTAMP no campo {0}, {1}"
            },
            "DOES_NOT_EXIST_FIELD": {
                "code": 34563456,
                "message": "O campo '{0}' não existe no microserviço.",
                "message_except": "ERRO FIELD campo {0}, {1}"
            },
            "DOES_NOT_EXIST_FIELD_TYPE": {
                "code": 5454542,
                "message": "No campo '{0}' o tipo configurado '{1}', não existe.",
                "message_except": "ERRO FIELD no campo {0}, {1}"
            },
            "MAX_LENGTH": {
                "code": 65734747,
                "message": "No campo '{0}' O tamanho máximo permitido é '{1}', foi recebido '{2}.",
                "message_except": "ERRO MAX_LENGTH no campo {0}, {1}"
            },
            "MIN_LENGTH": {
                "code": 4567567,
                "message": "No campo '{0}' O tamanho mínimo permitido é '{1}', foi recebido '{2}.",
                "message_except": "ERRO MIN_LENGTH no campo {0}, {1}"
            },
            "MIN_VALUE": {
                "code": 4564568,
                "message": "No campo '{0}' O valor mínimo permitido é '{1}', foi recebido '{2}.",
                "message_except": "ERRO MIN_VALUE no campo {0}, {1}"
            },
            "MAX_VALUE": {
                "code": 7685685,
                "message": "No campo '{0}' O valor máximo permitido é '{1}', foi recebido '{2}.",
                "message_except": "ERRO MAX_VALUE no campo {0}, {1}"
            },
            "FIELD_REQUIRED": {
                "code": 8484545,
                "message": "O campo '{0}' é requerido.",
                "message_except": "ERRO FIELD_REQUIRED no campo {0}, {1}"
            },
            "FALHA_DE_SERVICE": {
                "code": 6545454,
                "message": "Identificador do serviço nao valido:: '{0}' != '{1}' :: '{2}' != '{3}'.",
                "message_except": "ERRO FALHA DE SERVICE {0}, {1}"
            },
            "MAX_LIMIT_EXCEEDED": {
                "code": 85545454,
                "message": "O limite máximo ({0}) foi excedido. Limite solicitado ({1}).",
                "message_except": "ERRO MAX LIMIT EXCEEDED {0}, {1}"
            },
            "INVALID_ENDPOINT": {
                "code": 564564654,
                "message": "Endpoint não registrado '{0}' no microserviço '{1}' para o método '{2}'.",
                "message_except": "ERRO INVALID_ENDPOINT {0}, {1}"
            },
            "INVALID_ACTION": {
                "code": 5646546517,
                "message": "Ação '{0}' não registrada no microserviço '{1}' - '{2}' - para o método '{3}'.",
                "message_except": "ERRO INVALID ACTION {0}, {1}"
            },
            "INVALID_ACTION_DB": {
                "code": 654654654,
                "message": "Ação '{0}' não disponível para interações base de dados.",
                "message_except": "ERRO INVALID ACTION DB {0}, {1}"
            },
            "FIELD_CONFLICT": {
                "code": 5646546517,
                "message": "O campo '{0}' nao pode ser utilizado por existir campos filhos.",
                "message_except": "ERRO FIELD CONFLICT {0}, {1}"
            },
            "NO_DATA_TO_INSERT": {
                "code": 545645645,
                "message": "Não foi enviado dados para ser inserido na base de dados.",
                "message_except": "ERRO NO DATA TO INSERT {0}, {1}"
            },
            "NO_DATA_TO_UPDATE": {
                "code": 34545645632,
                "message": "Não foi enviado dados para ser atualizao na base de dados.",
                "message_except": "ERRO NO DATA TO UPDATE {0}, {1}"
            },
            "INVALID_SCHEME_UPDATE": {
                "code": 645456653,
                "message": "Esquema de update Inválido não encontrado $set, $inc...",
                "message_except": "ERRO INVALID SCHEME UPDATE {0}, {1}"
            },
            "WARN_FIND_ITEM": {
                "code": 5356456445,
                "message": "O filtro da busca encontrou mais mais itens do que o esperado, nenhum item é retornado.",
                "message_except": "WARN FIND ITEM {0}, {1}"
            },
            "WARN_INSERT_ITEM": {
                "code": 5356456445,
                "message": "Muitos documentos foram inseridos utilizando um endpoit do tipo 'item', somente o primeiro documento é retornado.",
                "message_except": "WARN INSERT ITEM {0}, {1}"
            },
            "SERVICE_INVALID": {
                "code": 66565,
                "message": "Microservice parâmetro inválido :: {1} em {0}.",
            },
            "MICROSERVICE_INVALID": {
                "code": 36645764,
                "message": "Microservice inválido :: Service ID:: {1}.",
            },
            "CONFIGURATION_SERVICE": {
                "code": 2454545454,
                "message": "{0}"
            }

        }

    # --------------------------------
    # addx
    # --------------------------------
    def addx(self, code, value, params):
        error = self.error_messages[code]
        self.add(error["code"], error["message"].format(*params))

        if (value is not None) and (type(value) == dict) and ("error" in value.keys()):
            params = [(params[1]), (value["error"])]
            self.add(error["code"], error["message_except"].format(*params))

    def alerts(self, code, value, params):
        self.__has_error = True
        self.addx(code, value, params)

    def error(self, code, value, params):
        self.__has_error = True
        self.addx(code, value, params)

    def info(self, code, value, params):
        self.__has_info = True
        self.addx(code, value, params)

    def log(self, code, value, params):
        self.__has_log = True
        self.addx(code, value, params)

    def warn(self, code, value, params):
        self.__has_warn = True
        self.addx(code, value, params)

    # --------------------------------
    # add_errors
    # --------------------------------
    # Adiciona uma lista de erros na lista de erros retornada
    #
    def add_errors(self, list_errors):
        if list_errors is not None:
            for error in list_errors:
                self.add(error['code'], error['message'])

    def add(self, code, message):
        self.__has_error = True
        self.__list_errors.append(
            {"code": code, "message": message}
        )

    def add_required(self, code, message):
        self.__has_error = True
        self.__list_errors.append(
            {"code": code, "message": message}
        )

    def add_field_required(self, field, code, message):
        self.__has_error = True
        self.__list_errors.append(
            {"field": field, "code": code, "message": message}
        )

    def to_list(self):
        if len(self.__list_errors) > 0:
            return self.__list_errors
        return None

    def ready(self):
        return self.__has_error is not True

    def has_error(self):
        return self.__has_error



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import ast
import bson
from datatypes import DataTypes


# TODO:: Refator completamente classe DataFilter
class DataFilter:
    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, params, status):
        list_params = self.__to_list(params)
        self.status = status
        self.labels = {"item": {}, "list": {}}

        self.__data_types = DataTypes()
        self.__params = {}
        self.__prepare(self.__params, list_params, "item")
        self.__conflict()

    # --------------------------------
    # __str__
    # --------------------------------
    def __str__(self):
        return self.__data_types.to_str(self.__params)

    # --------------------------------
    # add
    # --------------------------------
    # TODO:: implementar self.__data_types = DataTypes()
    def add(self, field, value):
        if type(value) == str:
            prepare_filter = self.__data_types.to_str(field + "(" + value + ")")
        else:
            prepare_filter = unicode(field + "(" + value + ")")

        self.__prepare(self.__params, [prepare_filter], 'item')

    # --------------------------------
    # check_options_values
    # --------------------------------
    @staticmethod
    def check_options_values(key, value):
        options = ["in", "nin", "and", "or", "nor"]
        if key in options:
            if type(value) != list:
                value = [value]

        count = 0
        for item in value:
            # { name: { $regex: "s3", $options: "si" } }
            # Verifica se é do tipo regex
            if type(item) == str or type(item) == unicode and re.match(r"/([\s\S]+)/([\s\S]+)\Z", item, re.IGNORECASE):
                regex = re.match(r"/([\s\S]+)/([\s\S]+)\Z", item, re.IGNORECASE)
                value[count] = {
                    "$regex": regex.group(1),
                    "$options": regex.group(2)
                }
                count = count + 1

        return value

    # --------------------------------
    # is_empty
    # --------------------------------
    # Verifica se o valor do campo não está vazio ou é do tipo js null
    @staticmethod
    def is_empty(item):
        # Verifica se o campo é diferente de string e se não está vazio
        if (len(item) == 0) or (item == ''):
            return False

    # --------------------------------
    # to_list
    # --------------------------------
    def to_list(self):
        return self.__params

    # --------------------------------
    # __conflict
    # --------------------------------
    def __conflict(self):
        fields_errors = []

        # Processa a lista de campos default
        for item_check in self.labels["item"]:
            # Retira um item da lista para comparar com outros itens da mesma lista
            for item_field in self.labels["item"]:
                # Caso um campo esteja em conflito com outros campos é adicionado na lista de erros
                if item_check + '.' in item_field:
                    fields_errors.append([item_check, item_field])
        if len(fields_errors) > 0:
            self.status.warn(3363569, "ERROR:: CAMPOS EM CONFLITOS" + self.__data_types.to_str(fields_errors))

    # --------------------------------
    # __dot_to_dict
    # --------------------------------
    def __dot_to_dict(self, d, dot_string, value):
        if dot_string == "id":
            dot_string = "_id"
        options = ["eq", "gt", "gte", "lt", "lte", "ne", "exists", "not", "type", "and", "or", "nor", "in", "nin"]
        if "." in dot_string:
            key, rest = dot_string.split(".", 1)
            if key == "id":
                key = "_id"
            if key not in d:
                key = key if key not in options else '$' + key
                d[key] = {}
            self.__dot_to_dict(d[key], rest, value)
        elif dot_string in options:
            if type(d) == dict:
                key = '$' + dot_string
                d[key] = self.check_options_values(dot_string, value)
        else:
            d[dot_string] = value

    # --------------------------------
    # __prepare
    # --------------------------------
    def __prepare(self, group_params, list_params, type_params):
        for param in list_params:
            regex_comparison_query_perators = r"(?:([\s\S]*|\.(eq|gt|gte|lt|lte|ne|exists|not|in|nin|type))\(([\s\S]+)\))\Z"
            match_comparison_query_perators = re.match(regex_comparison_query_perators, param, re.IGNORECASE)

            regex_comparison_query_perators_and_nor = r"(?:(and|nor)\[([\s\S]*)\])\Z"
            match_comparison_query_perators_and_nor = re.match(regex_comparison_query_perators_and_nor, param, re.IGNORECASE)

            regex_comparison_query_perators_or = r"(or)\{([\s\S]*)\}\Z"
            match_comparison_query_perators_or = re.match(regex_comparison_query_perators_or, param, re.IGNORECASE)

            if match_comparison_query_perators:
                group_filter = match_comparison_query_perators.group(1)
                group_value = match_comparison_query_perators.group(3)

                if re.match(r"^([a-z0-9_]+\.?)+$", group_filter, re.IGNORECASE):
                    self.labels[type_params][group_filter] = True
                    self.__dot_to_dict(group_params, group_filter, self.__values(group_value))
                else:
                    self.status.warn(87541545, "ERROR - Filtro inválido" + self.__data_types.to_str(group_filter))

            elif match_comparison_query_perators_and_nor:
                group_filter = match_comparison_query_perators_and_nor.group(1)
                group_value = match_comparison_query_perators_and_nor.group(2)

                if not re.match(r"(?:[a-z0-9]+\([\s\S]+\)[^+])\Z", group_value, re.IGNORECASE):
                    if re.match(r"^([a-z0-9_]+\.?)+$", group_filter, re.IGNORECASE):
                        group = group_params["$" + group_filter] = []
                        self.__values_list(group, group_value, "+")
                    else:
                        self.status.warn(523152, "ERROR FILTRO- de separador '|' nao encontrado" + self.__data_types.to_str(group_filter))

                else:
                    self.status.warn(523152, "ERROR FILTRO- de separador '|' nao encontrado" + self.__data_types.to_str(group_filter))

            elif regex_comparison_query_perators_or:
                group_filter = match_comparison_query_perators_or.group(1)
                group_value = match_comparison_query_perators_or.group(2)

                if not re.match(r"(?:[a-z0-9]+\([\s\S]+\)[^|])\Z", group_value, re.IGNORECASE):
                    if re.match(r"^([a-z0-9_]+\.?)+$", group_filter, re.IGNORECASE):
                        group = group_params["$" + group_filter] = []
                        self.__values_list(group, group_value, "|")
                    else:
                        self.status.warn(5454545454, "Filtro inválido" + self.__data_types.to_str(group_filter))

                else:
                    self.status.warn(523152, "ERROR FILTRO- de separador '|' nao encontrado" + self.__data_types.to_str(group_filter))

            else:
                self.status.warn(5454545454, "Filtro inválido XXXXXXX")

    # --------------------------------
    # __to_list
    # --------------------------------
    def __to_list(self, params, sep=";"):
        list_params = []
        if type(params) == str or type(params) == unicode:
            params = params.split(sep)
            for item in params:
                if self.is_empty(item) is not False:
                    list_params.append(item)
        return list_params

    # --------------------------------
    # __values
    # --------------------------------
    def __values(self, value):

        # Verifica se o valor é do tipo número
        if value.isdigit() or value.replace('.', '', 1).isdigit():
            value = ast.literal_eval(value)

        # Verifica se o valor é do tipo array
        elif "," in value:
            value = value.split(",")
            i = 0
            for item in value:
                if item.isdigit() or item.replace('.', '', 1).isdigit():
                    value[i] = ast.literal_eval(item)
                    i += 1
        # Verifica se é do tipo bolean
        elif value.lower() in ["true", "1"]:
            value = True
        elif value.lower() in ["false", "0"]:
            value = False

        # Verifica se é do tipo objectID
        elif re.match("^[0-9a-f]{24}$", value):
            try:
                value = bson.objectid.ObjectId(self.__data_types.to_str(value))
            except Exception as inst:
                value = {"error": inst.args[0]}

        # Verifica se é do tipo date
        # Verifica se é do tipo timestamp
        # Verifica se é uma lista

        if type(value) == list:
            count = 0
            # Processa s lista para verificar se os elementos são do tipo ObjectId
            for item in value:
                # Verifica se é do tipo objectId
                if re.match("^[0-9a-f]{24}$", item):
                    try:
                        # Converte string em ObjectId
                        value[count] = bson.objectid.ObjectId(self.__data_types.to_str(item))
                        count = count + 1
                    except Exception as inst:
                        value = {"error": inst.args[0]}

        return value

    # --------------------------------
    # __values_list
    # --------------------------------
    def __values_list(self, group_params, values, sep):
        list_params = self.__to_list(values, sep)
        if len(list_params) > 0:
            for item in list_params:
                item_filter = {}
                if re.match(r"(?:([\s\S]*|\.(eq|gt|gte|lt|lte|ne|exists|not|in|nin|type))\(([\s\S]+)\))\Z", item, re.IGNORECASE):
                    self.__prepare(item_filter, [item], "list")
                elif re.match(r"(or){([\s\S]*)}\Z", item, re.IGNORECASE):
                    list_items = item.split(sep)
                    self.__prepare(item_filter, list_items, "list")

                if len(item_filter.keys()) > 0:
                    group_params.append(item_filter)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import datetime
import re
import hashlib
from bson.json_util import dumps
from bson.json_util import loads
from validate import Validate
from datafilter import DataFilter
from datatypes import DataTypes
from decode_types import DecodeTypes


# TODO:: Refatorar completamente a classe DataRest
# TODO:: Implementar configuração de limites definido no console para o serviço
class DataRest:
    def __init__(self, message, custom_service, status):

        self.status = status

        # self.action = None
        self.database_id = None
        self.database_name = None
        self.collection_name = None
        self.body = None
        # self.data = {}
        # self.fields = []
        self.filter = {}
        self.group = None
        self.id = None
        # self.info = None
        # self.limit = 0
        self.query = None
        self.service = None
        self.service_id = None
        self.token = None
        self.version = None

        self.__action = None
        self.__data_base = None
        self.__data_base_id = None
        self.__changed_time = 0
        self.__checksum = None
        self.__collection = None
        self.__connection_id = None
        self.__created_time = 0
        self.__data = {}
        self.__data_types = DataTypes()
        self.__default_limit = 100
        self.__fields = []
        self.__info = {}
        self.__filter = None
        self.__limit = 0
        self.__max_limit = 200
        self.__method = None
        self.__offset = 0
        self.__regex = None
        self.__result_type = None
        self.__search = None
        self.__service = None
        self.__service_id = None
        self.__sort = None
        self.__source = None
        self.__token = None
        self.__version = None

        self.validate = Validate(custom_service, self.status)
        self.decode_type = DecodeTypes(self.status)
        self.decode(message)

    # --------------------------------
    # action
    # --------------------------------
    def action(self):
        return self.__action

    # --------------------------------
    # action_db
    # --------------------------------
    def action_db(self, action):
        self.__action = action

    # --------------------------------
    # changed_time
    # --------------------------------
    # TODO:: Implementar nos documentos atualizados
    def changed_time(self):
        self.__changed_time = self.__data_types.to_str(datetime.datetime.utcnow())

    # --------------------------------
    # checksum
    # --------------------------------
    # TODO:: Implementar checksum no docuemntos para insert e update
    def checksum(self, data):
        md5 = hashlib.md5()
        md5.update(dumps(data))
        self.__checksum = md5.hexdigest()

    # --------------------------------
    # collection
    # --------------------------------
    def collection(self, collection):
        if collection != self.__service:
            list_names = collection.split("_")
            if list_names[0] != self.__service:
                list_names.insert(0, self.__service)
                collection = '_'.join(list_names)
        self.__collection = collection

    # --------------------------------
    # created_time
    # --------------------------------
    def created_time(self):
        self.__created_time = self.__data_types.to_str(datetime.datetime.utcnow())

    def data(self, data):
        self.__data = data
        # Registra o checksum do documento
        self.checksum(data)

    # --------------------------------
    # decode_service
    # --------------------------------
    # TODO:: Mudar sistema de error, tirar de dentro do validate
    # TODO:: Mudar regra de criação de nomes de banco de dados
    def decode(self, message):
        # Prepara o pacote a ser enviado
        json = loads(message)

        self.body = json["body"]

        self.__action = self.decode_type.action(json["action"])
        self.__connection_id = self.decode_type.connection_id(json["connection_id"])
        self.__fields = json["fields"]
        self.__filter = DataFilter(json["filter"], self.status)
        self.__limit = int(json["limit"])
        self.__info = json["info"]
        self.__method = json["method"]
        self.__offset = int(json["offset"])
        self.__regex = json["regex"]
        self.__search = json["search"]
        self.__service = json["service"]
        self.__service_id = int(json["service_id"])
        self.__sort = json["sort"]
        self.__source = self.__set_source(json["source"])
        self.__token = json["token"]
        self.__version = json["version"]

        self.__data_base_id = self.__service_id + 20000
        # self.group = self.info["application"]["group"]
        self.__data_base = "db" + self.__data_types.to_str(self.__data_base_id) + self.__service
        # self.info["database"] = {"name": self.database_name, "collection": None}

    # --------------------------------
    # field_insert
    # --------------------------------
    def field_insert(self, item_scheme_name, item_scheme, value, action=None):
        return self.validate.field_insert(item_scheme_name, item_scheme, value, action)

    # --------------------------------
    # field_update
    # --------------------------------
    def field_update(self, item_scheme_name, item_scheme, value, action=None):
        return self.validate.field_update(item_scheme_name, item_scheme, value, action)

    # --------------------------------
    # fields
    # --------------------------------
    def fields(self, fields):
        # Verifica se recebeu uma lista de campos do usuário
        # Verifica se campos são válidos
        # Registra lista de campos solicitados
        if len(self.__fields) > 0:
            list_fields = []
            for item in self.__fields:
                if item in fields:
                    list_fields.append(item)
            self.__fields = list_fields
        else:
            # Registra lista de campos default
            self.__fields = fields

    # --------------------------------
    # info
    # --------------------------------
    # TODO:: Implementar info nos documentos inseridos e atualizados
    def info(self):
        return {
            "owner": {
                "id": self.__token,
                "last_id": self.__token
            }
        }

    # --------------------------------
    # limit
    # --------------------------------
    # Define o limite de documentos retornados
    #
    # ?limit=20
    #
    def limit(self, max_limit=0):
        # Condigura o limite default do serviço
        limit = self.__default_limit

        # Verifica se o limit da ação foi definido
        if self.__limit > 0:
            # Recupera o limite da operação para validar
            limit = error_limit = self.__limit

            # Verifica se o limite da operação maior do que o maximo permitido pelo serviço
            if limit > self.__max_limit:

                # Caso seja, então registra o limite maximo permitido
                limit = error_max_limit = self.__max_limit

                # Verifica se um limite máximo foi definido pelo desenvolvedor do serviço
                if (max_limit > 0) and (limit > max_limit):
                    # Caso seja, então registra o limite maximo permitido
                    error_max_limit = max_limit
                    limit = max_limit

                # Registra mensagem de erro no serviço de log
                self.status.warn("MAX_LIMIT_EXCEEDED", None,
                                 [self.__data_types.to_str(error_max_limit), self.__data_types.to_str(error_limit)])

        self.__limit = limit

    # --------------------------------
    # method
    # --------------------------------
    def method(self):
        return self.__method

    # --------------------------------
    # owner_id
    # --------------------------------
    # Armazena na _info o token do dono do documento
    #
    def owner_id(self):
        self.__info["owner"] = {
            "id": self.__token
        }

    # --------------------------------
    # owner_last_id
    # --------------------------------
    # Armazena na _info o último token do usuário que alterou o documento
    #
    def owner_last_id(self):
        self.__info["owner"] = {
            "last_id": self.token
        }

    # --------------------------------
    # result_type
    # --------------------------------
    def result_type(self, result_type):
        self.__result_type = result_type

    # --------------------------------
    # send_database
    # --------------------------------
    # TODO:: Implementar melhorias na verificação de disponibilidade do serviço para serguir adiante, chegando todos os tipos que serão enviados
    # TODO:: Verificar se os dados para serem enviados ao serviço de banco de dados estão dentro das especificações esperadas
    def send_database(self):
        message = {
            "action": self.__action,
            "collection": self.__collection,
            "connection_id": self.__connection_id,
            "data": self.__data,
            "database": self.__data_base,
            "errors": self.status.to_list(),
            "fields": self.__fields,
            "filter": self.__filter.to_list(),
            "limit": self.__limit,
            "offset": self.__offset,
            "method": self.__method,
            "result_type": self.__result_type,
            "sort": self.__sort,
            "service_id": self.__data_base_id,
            "info": self.info()
        }

        # Montar mensagem que será enviada para o serviço de banco de dados
        return dumps(message)

    # --------------------------------
    # send_error
    # --------------------------------
    # Montar mensagem de erro para ser retornada ao servidor rest
    # TODO: Preparar filtro para ser retornado ao cliente
    def send_error(self):
        return dumps({
            "connection_id": self.__connection_id,
            "message": {
                "success": False,
                "message": "Falha de interação com microserviço",
                "elapsed_time": {"response": "17.000", "server": "7.000"},
                "errors": self.status.to_list(),
                "query": {
                    "limit": self.__limit,
                    "offset": self.__offset,
                    "fields": self.__fields,
                    "filter": self.__filter.to_list()
                },
            }
        })

    # --------------------------------
    # source
    # --------------------------------
    def source(self):
        return self.__source

    # --------------------------------
    # validate_service
    # --------------------------------
    # Testa a validade do serviço
    #
    def validate_service(self, service, service_id):
        if (self.__service != service) or (self.__data_types.to_str(self.__service_id)) != (self.__data_types.to_str(service_id)):
            # Registra mensagem de erro no serviço de log
            self.status.error("FALHA_DE_SERVICE", None, [self.__service, service, self.__data_types.to_str(
                self.__service_id), self.__data_types.to_str(service_id)])
            return False
        return True

    # --------------------------------
    # __filter_extras
    # --------------------------------
    # Todo Finalizar sistema de seach e regex
    def __filter_extras(self):
        # self.__prepare_regex()
        self.__prepare_search()

    # --------------------------------
    # __prepare_regex
    # --------------------------------
    def __prepare_regex(self):
        print self.__regex

    # --------------------------------
    # __prepare_search
    # --------------------------------
    # { name: { $regex: "s3", $options: "si" } }
    # or{quantity.lt(20)|price(10)}
    # tags.in( '{ $regex: "s3", $options: "si" }', '{ $regex: "s3", $options: "si" }')
    def __prepare_search(self):
        values, fields = self.__search.split(';')
        values = values.split()
        fields = fields.split(',')
        if len(fields) > 0:
            for item in fields:
                for word in values:
                    # self.__filter.add("_id.eq", result.group(2))
                    print "------->", word

    # --------------------------------
    # __set_source
    # --------------------------------
    # Recupera o id do documento
    def __set_source(self, source):
        # Verificar se o source possui uma id
        # Pegar a id do source
        result = re.search(r"([\s\S]+)([0-9a-f]{24})$", source)
        if result:
            source = result.group(1) + ":id"
            self.__filter.add("_id.eq", result.group(2))

        return source



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from bson.json_util import dumps
from bson.json_util import loads


class DataRestDB:
    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self, message, status):
        self.__action = None
        self.__collection = None
        self.__connection_id = None
        self.__data = {}
        self.__data_base = None
        self.__fields = []
        self.__result_type = None
        self.__sort = {}
        self.__service_id = None
        self.__info = {}
        self.__filter = {}
        self.__limit = 0
        self.__offset = 0
        self.__message_send = {}
        self.__method = None
        self.__token = None

        self.status = status
        self.decode(message)

    # --------------------------------
    # action
    # --------------------------------
    def action(self):
        return self.__action

    # --------------------------------
    # collection
    # --------------------------------
    def collection(self):
        return self.__collection

    # --------------------------------
    # data
    # --------------------------------
    def data(self):
        return self.__data

    # --------------------------------
    # database
    # --------------------------------
    def database(self):
        return self.__data_base

    # --------------------------------
    # decode
    # --------------------------------
    # TODO:: Verificar todas as entradas de dados, confirmar tipagem esperada
    # TODO:: Implementar configurações do info de documento
    def decode(self, message):
        # Prepara o pacote a ser enviado
        json = loads(message)

        print "DECODE DB---->>>", json

        self.__action = json["action"]
        self.__collection = json["collection"]
        self.__connection_id = json["connection_id"]
        self.__data = json["data"]
        self.__data_base = json["database"]
        self.__fields = json["fields"]
        self.__result_type = json["result_type"]
        self.__sort = json["sort"]
        self.__service_id = json["service_id"]
        self.__info = json["info"]
        self.__filter = json["filter"]
        self.__limit = json["limit"]
        self.__offset = json["offset"]
        self.__method = json["method"]
        # self.__token = json["token"]
        self.status.add_errors(json["errors"])

    # --------------------------------
    # fields
    # --------------------------------
    def fields(self):
        return self.__fields

    # --------------------------------
    # filter
    # --------------------------------
    def filter(self):
        return self.__filter

    # --------------------------------
    # info
    # --------------------------------
    def info(self):
        return {
            "owner": {
                "id": self.__token,
                "last_id": self.__token
            }
        }

    # --------------------------------
    # limit
    # --------------------------------
    def limit(self):
        return self.__limit

    # --------------------------------
    # offset
    # --------------------------------
    def offset(self):
        return self.__offset

    # --------------------------------
    # result
    # --------------------------------
    # TODO:: Melhorar formato de dados retornados ao cliente
    def result(self, message):
        message["elapsed_time"] = {"response": "17.000", "server": "7.000"}
        message["method"] = self.__method

        if self.status.has_error():
            message["errors"] = self.status.to_list()

        self.__message_send = message

    # --------------------------------
    # result_type
    # --------------------------------
    def result_type(self):
        return self.__result_type

    # --------------------------------
    # send
    # --------------------------------
    # TODO:: Verificar se os dados para serem enviados ao cliente estão dentro das especificações esperadas
    # TODO:: Implementar sistema de gestão de consumo, analitico, histórico e log
    def send(self):
        # Registrar no stack de log
        # Registrar no stack de analytic
        # Registrar no stack de error
        # Registrar no stack de history
        # self.analyric.add({})

        return dumps({
            "connection_id": self.__connection_id,
            "message": self.__message_send
        })

    # --------------------------------
    # send_error
    # --------------------------------
    # TODO:: Melhorar mensagem de retorno em caso de falha
    def send_error(self):
        # Montar mensagem que será enviada para o serviço de banco de dados
        return dumps({
            "connection_id": self.__connection_id,
            "errors": self.status.to_list()
        })

    # --------------------------------
    # sort
    # --------------------------------
    def sort(self):
        return self.__sort



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import socket

HOST_NAME = socket.gethostname()
HOST_IP = socket.gethostbyname(HOST_NAME)
BROKER_IP = 'localhost'
if HOST_IP[:3] == '10.':
    BROKER_IP = '10.10.0.1'

print 'HOSTNAME:', HOST_NAME
print 'HOST IP:', HOST_IP
print 'BROKER_IP:', BROKER_IP




BROKER_DATABASE_ENDPOINT_A = 'tcp://localhost:41234'
BROKER_DATABASE_ENDPOINT_B = 'tcp://localhost:41640'
BROKER_DATABASE_ENDPOINT_C = 'tcp://localhost:43210'
MONGO_IP = 'localhost'
if HOST_IP[:3] == '10.':
    BROKER_DATABASE_ENDPOINT_A = 'tcp://10.30.0.1:41234'
    BROKER_DATABASE_ENDPOINT_B = 'tcp://10.30.0.1:41640'
    BROKER_DATABASE_ENDPOINT_C = 'tcp://10.30.0.1:43210'
    MONGO_IP = '10.50.0.1'

print 'HOSTNAME:', HOST_NAME
print 'HOST IP:', HOST_IP
print 'BROKER_DATABASE_ENDPOINT_A:', BROKER_DATABASE_ENDPOINT_A
print 'BROKER_DATABASE_ENDPOINT_B:', BROKER_DATABASE_ENDPOINT_B
print 'BROKER_DATABASE_ENDPOINT_C:', BROKER_DATABASE_ENDPOINT_C
print 'MONGO_IP:', MONGO_IP

server_console = {
    "5abe46fd83566949ecc10121": [
        {
            "id": "7b85814a9f68d0ad190c4d84",
            "name": "domains",
            "ip": BROKER_IP,
            "service_id": "21234",
            "connections": 0,
            "errors": 0
        },
        {
            "id": "7b85814a9f68d0ad190c4d84",
            "name": "log",
            "ip": BROKER_IP,
            "service_id": "23210",
            "connections": 0,
            "errors": 0
        },
        {
            "id": "7b85814a9f68d0ad190c4d84",
            "name": "dental",
            "ip": BROKER_IP,
            "service_id": "21640",
            "connections": 0,
            "errors": 0
        }
    ],
    "5abe46fd83566949ecc30188": [
        {
            "id": "7b85814a9f68d0ad190c4d86",
            "name": "domains",
            "service_id": "41234",
            "connections": 0,
            "errors": 0
        },
        {
            "id": "7b85814a9f68d0ad190c4d86",
            "name": "log",
            "service_id": "43210",
            "connections": 0,
            "errors": 0
        },
        {
            "id": "7b85814a9f68d0ad190c4d86",
            "name": "dental",
            "service_id": "41640",
            "connections": 0,
            "errors": 0
        }
    ],

    "41234":
        {
            "id": "9885814a9f68d0ad190c4d79",
            "name": "domains",
            "broker_database_endpoint": BROKER_DATABASE_ENDPOINT_A,
            "mongodb_ip": MONGO_IP,
            "mongodb_port": 27017,
            "service_id": "41234",
            "connections": 0,
            "errors": 0
        },
    "41640":
        {
            "id": "a885814a9f68d0ad190c4d7a",
            "name": "dental",
            "broker_database_endpoint": BROKER_DATABASE_ENDPOINT_B,
            "mongodb_ip": MONGO_IP,
            "mongodb_port": 27017,
            "service_id": "41640",
            "connections": 0,
            "errors": 0
        },
    "43210":
        {
            "id": "a885814a9f68d0ad190c4d7a",
            "name": "log",
            "broker_database_endpoint": BROKER_DATABASE_ENDPOINT_C,
            "mongodb_ip": MONGO_IP,
            "mongodb_port": 27017,
            "service_id": "43210",
            "connections": 0,
            "errors": 0
        }
}


import socket
HOST_NAME = socket.gethostname()
HOST_IP = socket.gethostbyname(HOST_NAME)
BROKER_REST_ENDPOINT = 'tcp://localhost:23210'
BROKER_DATABASE_ENDPOINT = 'tcp://localhost:30188'

# Verifica se esta executando no Docker ou On-line
if HOST_IP[:3] == '10.':
    BROKER_REST_ENDPOINT = 'tcp://10.10.0.1:23210'
    BROKER_DATABASE_ENDPOINT = 'tcp://10.30.0.1:30188'

print 'HOSTNAME:', HOST_NAME
print 'HOST IP:', HOST_IP
print 'BROKER_REST_ENDPOINT:', BROKER_REST_ENDPOINT
print 'BROKER_DATABASE_ENDPOINT:', BROKER_DATABASE_ENDPOINT


server_console_scheme = {
    "23210":
        {
            "uid": "4885814a9f68d0ad190c4d76",
            "id": "23210",
            "name": "log",
            "broker_rest_endpoint": BROKER_REST_ENDPOINT,
            "broker_database_endpoint": BROKER_DATABASE_ENDPOINT,
            "service_id": "23210",
            "errors": 0,

            "schemes": {
                "type_array": {"type": "array", "info": "Array", "version": "1.0"},
                "type_boolean": {"type": "boolean", "info": "Boolean", "version": "1.0"},
                "type_number": {"type": "number", "info": "Number", "version": "1.0", "min_value": "100", "max_value": "3000", "required": "true"},
                "type_decimal": {"type": "decimal", "info": "Number Decimal", "version": "1.0"},
                "type_object": {"type": "object", "info": "Object", "version": "1.0"},
                "type_timestamp": {"type": "timestamp", "info": "Timestamp", "version": "1.0"},
                "type_objectid": {"type": "object_id", "info": "ObjectId", "version": "1.0"},
                "type_enum": {"type": "enum", "info": "Enum", "version": "1.0", "enum": ["M", "F", "34"]},
                "type_string": {"type": "string", "info": "String", "version": "1.0"},
                "type_date": {"type": "date", "info": "Date", "version": "1.0"},
                "type_bola": {"type": "bola", "info": "Bola", "version": "1.0"},

                "code": {"type": "number", "info": "Código do erro", "version": "1.0"},
                "script": {"type": "object", "info": "Nome do script", "version": "1.0"},
                "container": {"type": "object", "info": "ID do container", "version": "1.0"},
                "instance": {"type": "object", "info": "ID da instância", "version": "1.0"},
                "created_at": {"type": "date", "info": "Horário da ocorrência", "version": "1.0"},
                "user": {"type": "object", "info": "Código do usuário", "version": "1.0"},
                "service": {"type": "object", "info": "Código do serviço", "version": "1.0"},
                "type": {"type": "string", "info": "Tipo do (error|warn|info|log)", "version": "1.0"},
                "message": {"type": "string", "info": "Detalhes extras", "version": "1.0"}
            },

            "actions": {
                "get": {
                    # --------------------------------
                    # messages
                    # --------------------------------
                    "/1.0/devdoo/log/system/messages": {
                        "action": {
                            "scheme": [
                                "type_array",
                                "type_boolean",
                                "type_number",
                                "type_decimal",
                                "type_object",
                                "type_timestamp",
                                "type_objectid",
                                "type_enum",
                                "type_string",
                                "type_date",
                                "type_bola",
                                "code",
                                "script",
                                "container",
                                "created_at",
                                "user",
                                "service",
                                "type",
                                "message"
                            ],
                            "database_action": "find",
                            "collection": "messages",
                            "result_type": "list"
                        }
                    }
                },
                "put": {},
                "post": {
                    # --------------------------------
                    # mensagens
                    # --------------------------------
                    "/1.0/devdoo/log/system/add-message": {
                        "action": {
                            "scheme": [
                                "type_array",
                                "type_boolean",
                                "type_number",
                                "type_decimal",
                                "type_object",
                                "type_timestamp",
                                "type_objectid",
                                "type_enum",
                                "type_string",
                                "type_date",
                                "type_bola",
                                "code",
                                "script",
                                "container",
                                "created_at",
                                "user",
                                "service",
                                "type",
                                "message"
                            ],
                            "database_action": "insert",  # TODO: Mudar o nome da operação de database para operation
                            "collection": "messages",
                            "result_type": "item"
                        }
                    }
                },
                "delete": {
                    # --------------------------------
                    # mensagens
                    # --------------------------------
                    "/1.0/devdoo/log/system/remove-message": {
                        "action": {
                            "scheme": [],
                            "collection": "operators",
                            "database_action": "remove",
                            "result_type": "list",
                        }
                    }
                }
            }
        }
}


class Console:

    def services_config(self, id):
        return server_console[id]


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pydash
from devdoo.datatypes import DataTypes


# TODO:: Verificar tipos inexistentes
class Validate(object):
    def __init__(self, custom_service, status):
        self.status = status
        self.__custom_service = custom_service

    # --------------------------------
    # array
    # --------------------------------
    # Valida campo do tipo array
    # Retorna array
    #
    # Exemplos: ["a", "b", 1, 2] | a,b,c,1,2
    #
    def array(self, scheme, value):
        # Converte valor para string
        original_value = value = DataTypes.to_str(value)

        # Verifica se o campo é obrigatório
        if self.__has_error_required(scheme, value):
            return '__error__'

        # Converte valor para array
        value = DataTypes.to_array(value)

        # Verifica se o resultado do tipo convertido é um array válido
        if not DataTypes.is_array(value):
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("TYPE_INVALID_ARRAY", value, [field, original_value])
            value = '__error__'

        return value

    # --------------------------------
    # boolean
    # --------------------------------
    # Valida campo do tipo boleano
    # Retorna boleano
    #
    # Exemplos: TRUE, True, true, 1, FALSE, False, false, 0
    #
    def boolean(self, scheme, value):
        # Converte valor para string
        original_value = value = DataTypes.to_str(value)

        # Verifica se o campo é obrigatório
        if self.__has_error_required(scheme, value):
            return '__error__'

        # Converte valor para boleano
        value = DataTypes.to_boolean(value)

        # Verifica se o resultado do tipo convertido é um boleano válido
        if not DataTypes.is_boolean(value):
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("TYPE_INVALID_BOOLEAN", value, [field, Validate.clean_value(original_value)])
            value = '__error__'
        return value

    # --------------------------------
    # date
    # --------------------------------
    # Valida campo do tipo data
    #
    # Exemplo: 9999-12-31T23:59:59.999
    # AAAA-MM-DDTHH:MM:SS.mmm
    # AAAA-MM-DD
    # Retorna Datetime
    #
    def date(self, scheme, value):
        # Converte valor para string
        original_value = value = DataTypes.to_str(value)

        # Verifica se o campo é obrigatório
        if self.__has_error_required(scheme, value):
            return '__error__'

        # Converte valor para boleano
        value = DataTypes.to_date(value)

        # Verifica se o resultado do tipo convertido é uma data válida
        if not DataTypes.is_date(value):
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("TYPE_INVALID_DATE", value, [field, Validate.clean_value(original_value)])
            value = '__error__'
        return value

    # --------------------------------
    # decimal (float)
    # --------------------------------
    # Valida campo do tipo número decimal
    # Retorna float decimal
    #
    # TODO:: Implementar min/max para decimal
    def decimal(self, scheme, value):
        # Converte valor para string
        original_value = value = DataTypes.to_str(value)

        # Verifica se o campo é obrigatório
        if self.__has_error_required(scheme, value):
            return '__error__'

        # Converte valor para número
        value = DataTypes.to_decimal(value)

        # Verifica se o resultado do tipo convertido é um decimal válido
        if DataTypes.is_decimal(value):
            pass
            # min_value / max_value
            # error_min = self.__min_value_decimal(field, value)
            # error_max = self.__max_value_decimal(field, value)
            # if error_min or error_max:
            #    value = '__error__'
        else:
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("TYPE_INVALID_DECIMAL", value, [field, Validate.clean_value(original_value)])
            value = '__error__'
        return value

    # --------------------------------
    # enum
    # --------------------------------
    # Valida campo do tipo opções de enumeração
    # Aceita somente opções string
    #
    # type: enum
    # enum:["M", "F", "34"]
    #
    def enum(self, scheme, value):
        # Converte valor para string
        original_value = value = DataTypes.to_str(value)

        # Verifica se o campo é obrigatório
        if self.__has_error_required(scheme, value):
            return '__error__'

        # Recupera a lista de opções do enumerador
        options = Validate.type(scheme, "enum")

        # Verifica se o valor é do tipo vazio ou none
        if (value == '__empty__') or (value == '__none__') or (value not in options):
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("TYPE_INVALID_ENUM", value, [field, Validate.clean_value(original_value), DataTypes.to_str(options)])
            value = '__error__'

        return value

    # --------------------------------
    # number
    # --------------------------------
    # Valida campo do tipo número longo
    #
    # type: number
    # min_value: 150
    # max_value: 300
    # formater:??
    #
    def number(self, scheme, value):
        # Converte valor para string
        original_value = value = DataTypes.to_str(value)

        # Verifica se o campo é obrigatório
        if self.__has_error_required(scheme, value):
            return '__error__'

        # Converte valor para número
        value = DataTypes.to_number(value)

        # Verifica se o resultado do tipo convertido é um número válido
        if DataTypes.is_number(value):
            # min_value / max_value
            error_min = self.__min_value(scheme, value)
            error_max = self.__max_value(scheme, value)
            if error_min or error_max:
                value = '__error__'
        else:
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("TYPE_INVALID_NUMBER", value, [field, Validate.clean_value(original_value)])
            value = '__error__'
        return value

    # --------------------------------
    # object
    # --------------------------------
    # Valida campo do tipo object
    #
    # type: object
    #
    def object(self, scheme, value):
        # Converte valor para string
        original_value = value = DataTypes.to_str(value)

        # Verifica se o campo é obrigatório
        if self.__has_error_required(scheme, value):
            return '__error__'

        # Converte valor para número
        value = DataTypes.to_object(value)

        # Verifica se o resultado do tipo convertido é objeto válido
        if not DataTypes.is_object(value):
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("TYPE_INVALID_OBJECT", value, [field, Validate.clean_value(original_value)])
            value = '__error__'
        return value

    # --------------------------------
    # object_id
    # --------------------------------
    # Valida campo do tipo objectId
    #
    # type: object_id
    # Exemplo: 5abb87da835669241478b102
    def object_id(self, scheme, value):
        # Converte valor para string
        original_value = value = DataTypes.to_str(value)

        # Verifica se o campo é obrigatório
        if self.__has_error_required(scheme, value):
            return '__error__'

        # Converte valor para número
        value = DataTypes.to_object_id(value)

        # Verifica se o resultado do tipo convertido é um object_id válido
        if not DataTypes.is_object_id(value):
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("TYPE_INVALID_OBJECT_ID", value, [field, Validate.clean_value(original_value)])
            value = '__error__'
        return value

    # --------------------------------
    # string
    # --------------------------------
    #
    # Valida campo do tipo string
    #
    # type: string
    # modifier: ["uppercase", "..."]
    # min_length: 5
    # max_length: 15
    # formater:??
    #
    def string(self, scheme, value):
        value = DataTypes.to_str(value)

        # Verifica se o campo é obrigatório
        if self.__has_error_required(scheme, value):
            return '__error__'

        # Remove elementos indesejados da string
        value = pydash.strings.escape(value).strip()
        value = Validate.clean_value(value)

        # min_length / max_length
        error_min = self.__min_length(scheme, value)
        error_max = self.__max_length(scheme, value)
        if error_min or error_max:
            return '__error__'

        # Implementa modificadores
        return Validate.modifier(scheme, value)

    # --------------------------------
    # timestamp
    # --------------------------------
    # Valida campo do tipo timestamp
    # Exemplo: 1522239066
    def timestamp(self, scheme, value):
        # Converte valor para string
        original_value = value = DataTypes.to_str(value)

        # Verifica se o campo é obrigatório
        if self.__has_error_required(scheme, value):
            return '__error__'

        # Converte valor para número
        value = DataTypes.to_timestamp(value)

        # Verifica se o resultado do tipo convertido é um timestamp válido
        if not DataTypes.is_timestamp(value):
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("TYPE_INVALID_TIMESTAMP", value, [field, Validate.clean_value(original_value)])
            value = '__error__'
        return value

    # --------------------------------
    # field
    # --------------------------------
    def __field(self, field_name, scheme, value, action=None):
        # Implementa valor default em caso do campo não receber valor do cliente

        field_value = Validate.default(scheme, value)
        field_type = scheme["type"]

        if hasattr(self, field_type):
            # Recupera o método do type que deverá ser executado
            method = getattr(self, field_type)
            field_value = method(scheme, field_value)
        else:
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("DOES_NOT_EXIST_FIELD_TYPE", value, [field, field_type])
            return field_name, '__error__'

        if action is not None:
            field_name, field_value = self.__custom(action, scheme, field_value, self.status)

        return field_name, field_value

    # --------------------------------
    # field_insert
    # --------------------------------
    def field_insert(self, item_scheme_name, scheme, value, action=None):
        field_name, field_value = self.__field(item_scheme_name, scheme, value, action)
        # Verifica se o nome do campo é dot
        if "." in field_name:
            field_name, field_value = Validate.field_dot(field_name, field_value)

        return field_name, field_value

    # --------------------------------
    # field_update
    # --------------------------------
    def field_update(self, item_scheme_name, scheme, value, action=None):
        return self.__field(item_scheme_name, scheme, value, action)

    # --------------------------------
    # default
    # --------------------------------
    @staticmethod
    def default(scheme, value):
        if (value == '__empty__') or (value is None):
            if "default" in scheme.keys():
                value = scheme["default"]
        return value

    # --------------------------------
    # clean_value
    # --------------------------------
    @staticmethod
    def clean_value(value):
        if value == '__empty__' or value == '__none__':
            value = ''
        return value

    # --------------------------------
    # check_empty
    # --------------------------------
    @staticmethod
    def check_empty(value):
        if (len(value) == 0) or (value == ''):
            value = '__empty__'
        elif value == 'null':
            value = '__none__'
        return value

    # --------------------------------
    # check_value
    # --------------------------------
    @staticmethod
    def check_value(value):
        if value == '__empty__' or value == '__none__':
            value = ''
        else:
            value = '__error__'
        return value

    # --------------------------------
    # lodash
    # --------------------------------
    # TODO:: Implementar outros modificadores
    # TODO:: Testar modificadores
    @staticmethod
    def lodash(action, value):
        if action == 'camel_case':
            value = pydash.strings.camel_case(value)
        elif action == 'clean':
            value = pydash.strings.clean(value)
        elif action == 'decapitalize':
            value = pydash.strings.decapitalize(value)
        elif action == 'escape':
            value = pydash.strings.escape(value)
        elif action == 'lowercase':
            value = pydash.strings.to_lower(value)
        elif action == 'kebab_case':
            value = pydash.strings.kebab_case(value)
        elif action == 'uppercase':
            value = pydash.strings.to_upper(value)
        elif action == 'upper_first':
            value = pydash.strings.upper_first(value)
        elif action == 'startcase':
            value = pydash.strings.start_case(value)
        elif action == 'titlecase':
            value = pydash.strings.title_case(value)
        elif action == 'unescape':
            value = pydash.strings.unescape(value)

        return value

    # --------------------------------
    # modifier
    # --------------------------------
    @staticmethod
    def modifier(field, value):
        modifier = Validate.type(field, "modifier")
        if modifier is not None:
            for action in modifier:
                value = Validate.lodash(action, value)
        return value

    # --------------------------------
    # type
    # --------------------------------
    @staticmethod
    def type(scheme, field):
        result = None
        if field in scheme.keys():
            result = scheme[field]
        return result

    # --------------------------------
    # field_dot
    # --------------------------------
    @staticmethod
    def field_dot(field_name, field_value):
        # Converte string em objetos
        def put(obj, dot_string, value):
            if "." in dot_string:
                key, rest = dot_string.split(".", 1)
                if key not in obj:
                    obj[key] = {}
                put(obj[key], rest, value)
            else:
                obj[dot_string] = value

        d = {}
        field_name_first, field_name = field_name.split(".", 1)
        put(d, field_name, field_value)
        return field_name_first, d

    # --------------------------------
    # __custom
    # --------------------------------
    def __custom(self, action, scheme, field_value, error):
        field_name = scheme["field"]
        print "CUSTOM", action, error, self.__custom_service
        '''
        if action in scheme.keys():
            # Verifica se a acao existe na instancia do servico
            if hasattr(self.__custom_service, scheme["field"] + "_" + action):
                method = getattr(self.__custom_service, scheme["field"] + "_" + action)
                field_name, field_value = method(field_name, field_value, error)
        '''
        return field_name, field_value

    # --------------------------------
    # __max_length
    # --------------------------------
    def __max_length(self, scheme, value):
        has_error = False
        max_length = Validate.type(scheme, "max_length")
        if (max_length is not None) and (len(value) > max_length):
            field = scheme["field"]
            has_error = True
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("MAX_LENGTH", value, [field, DataTypes.to_str(max_length), DataTypes.to_str(len(value))])
        return has_error

    # --------------------------------
    # __min_length
    # --------------------------------
    def __min_length(self, scheme, value):
        has_error = False
        min_length = Validate.type(scheme, "min_length")
        if (min_length is not None) and (len(value) < min_length):
            field = scheme["field"]
            has_error = True
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("MIN_LENGTH", value, [field, DataTypes.to_str(min_length), DataTypes.to_str(len(value))])
        return has_error

    # --------------------------------
    # __max_value
    # --------------------------------
    def __max_value(self, scheme, value):
        has_error = False
        max_value = Validate.type(scheme, "max_value")
        if value > DataTypes.to_number(max_value):
            field = scheme["field"]
            has_error = True
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("MAX_VALUE", value, [field, DataTypes.to_str(max_value), DataTypes.to_str(value)])
        return has_error

    # --------------------------------
    # __min_value
    # --------------------------------
    def __min_value(self, scheme, value):
        has_error = False
        min_value = Validate.type(scheme, "min_value")
        if value < DataTypes.to_number(min_value):
            field = scheme["field"]  # onde está a condição   valor < min_value
            has_error = True
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("MIN_VALUE", value, [field, DataTypes.to_str(min_value), DataTypes.to_str(value)])
        return has_error

    # --------------------------------
    # __has_error_required
    # --------------------------------
    def __has_error_required(self, scheme, value):
        is_required = Validate.type(scheme, "required")
        # Verifica se o campo e requerido
        if ((value == '__empty__') or (value == '')) and is_required:
            field = scheme["field"]
            # Registra mensagem de erro no serviço de log e retona ao cliente
            self.status.alerts("FIELD_REQUIRED", None, [field])
            return True
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import zmq
from service_db import ServiceDB


# TODO:: Implementar autoconfiguração do serviço, comunicação com console admin
class DevdooDatabase(object):
    # --------------------------------
    # __init__
    # --------------------------------
    def __init__(self):
        # Define endpoints de conexão
        broker_database_endpoint = service_config["broker_database_endpoint"]

        # TODO:: Melhorar forma como as configurações do banco de dados são implementadas
        self.mongodb_ip = service_config["mongodb_ip"]
        self.mongodb_port = service_config["mongodb_port"]

        # Cria o contexto ZMQ
        context = zmq.Context()

        # Adaptador de conexão
        self.socket_frontend = context.socket(zmq.REP)
        self.socket_frontend.connect(broker_database_endpoint)

        # Serviço a ser executado
        self.service = None

    # --------------------------------
    # console_service_list
    # --------------------------------
    # Obtém da console a lista de configuração
    #
    # TODO:: Remover console_service_list, implementar sistema de autoconfiguração
    def console_service_list(self, server_console, service_id):
        return server_console[service_id]

    # --------------------------------
    # operation
    # --------------------------------
    # Executa a ação do Serviço
    #
    def operation(self):
        # Recebe do mensagem do servidor cliente
        message = self.socket_frontend.recv()
        self.process(message)

        # Processa a operacao e envia mensage de retorno para o cliente
        self.socket_frontend.send(self.service.send())

    # --------------------------------
    # process
    # --------------------------------
    # Inicia o Serviço de banco
    #
    # TODO:: Mudar a forma como deverá ser configurado o serviço
    def process(self, message):
        self.service = ServiceDB(message, self.mongodb_ip, self.mongodb_port)

    # --------------------------------
    # run
    # --------------------------------
    # Executa a operação em um loop infinito
    #
    # TODO:: Implementar ping de verificação de disponibilidade do serviço
    def run(self):
        while True:
            try:
                self.operation()
            except Exception as inst:
                print "API_DB::RUN::ERROR", inst.args[0]
